#include "G4RunManager.hh"
#include "G4UImanager.hh"

#include "Randomize.hh"
#include "SBProba.hh"
#include "SBDetectorConstruction.hh"
//#include "SBPhysicsList.hh"
#include "QGSP.hh"
#include "SBPrimaryGeneratorAction.hh"
#include "SBRunAction.hh"
#include "SBEventAction.hh"
#include "SBSteppingAction.hh"
#include "SBSteppingVerbose.hh"
//#include "HistoManager.hh"
#include "MyTrackingAction.hh"
//#include "G4BlineTracer.hh"

//#include "DumpDataROOT.hh"

#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif

#if defined(G4UI_USE_TCSH)
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"
#elif defined(G4UI_USE_XM)
#include "G4UIXm.hh"
#elif defined(G4UI_USE_WIN32)
#include "G4UIWin32.hh"
#elif defined(G4UI_USE_QT)
#include "G4UIQt.hh"
#include "G4Qt.hh"
#else
#include "G4UIterminal.hh"
#endif


int main(int argc,char** argv)
{
  // Choose the Random engine
  //
  CLHEP::HepRandom::setTheEngine(new CLHEP::RanecuEngine);
  
  // User Verbose output class
  //
  G4VSteppingVerbose::SetInstance(new SBSteppingVerbose);
  //
  G4RunManager * runManager = new G4RunManager;
  //
  // Set mandatory initialization classes
  //
  SBDetectorConstruction* detector = new SBDetectorConstruction;
  runManager->SetUserInitialization(detector);
  //
  //G4VUserPhysicsList* physics = new SBPhysicsList;
  G4VUserPhysicsList* physics = new QGSP;
  runManager->SetUserInitialization(physics);
  //
  //HistoManager*  histo = new HistoManager();
  //DumpDataROOT* m_hh = new DumpDataROOT();
  // Set user action classes
  //
  //G4VUserPrimaryGeneratorAction* gen_action = new SBPrimaryGeneratorAction(detector);
  SBPrimaryGeneratorAction* gen_action = new SBPrimaryGeneratorAction(detector);
  runManager->SetUserAction(gen_action);
  // again
  //runManager->SetUserInitialization(detector);
  //runManager->SetUserInitialization(gen_action);
  //
  // to visualize smooth tracks in magnetic field
  //runManager->SetUserAction(new MyTrackingAction);

  //
  SBRunAction* run_action = new SBRunAction(gen_action);  
  runManager->SetUserAction(run_action);
  //
  SBProba *probab = new SBProba(detector,run_action);
  //
  SBEventAction* event_action = new SBEventAction(run_action,probab);
  runManager->SetUserAction(event_action);
  //
  G4UserSteppingAction* stepping_action = new SBSteppingAction(detector, event_action, gen_action, run_action);
  runManager->SetUserAction(stepping_action);

  //G4BlineTracer* theBlineTool = new G4BlineTracer();

  
  // Initialize G4 kernel
  //
  runManager->Initialize();
  
  //m_hh->init();

#ifdef G4VIS_USE
  // Initialize visualization
  //
  G4VisManager* visManager = new G4VisExecutive;
  visManager->Initialize();
#endif
  
  // Get the pointer to the User Interface manager
  //
  G4UImanager* UI = G4UImanager::GetUIpointer();      
 
  // andrea: implementare stepping ...
  // Antoine
  //100 KeV (muon, hadron) 
  //10 KeV (e+e-gamma)
  //10 mrad if B
  //100 micron and lose <1% Ek in conductors
  //

  // non questi!!
  // see http://geant4.web.cern.ch/geant4/UserDocumentation/UsersGuides/ForApplicationDeveloper/html/ch05s08.html#sect.G4Error.StepLimits
  //UI->ApplyCommand("/geant4e/limits/stepLength 0.1 mm");
  //UI->ApplyCommand("/geant4e/limits/energyLoss 0.001");
  //UI->ApplyCommand("/geant4e/limits/magField 0.001*tesla");
  
  int dum=0;
  G4cout << Form("\\rm -f g4_??.eps") << G4endl;
  dum=system(Form("\\rm -f g4_??.eps"));
  G4cout << Form("\\rm -f g4_??.prim") << G4endl;
  dum=system(Form("\\rm -f g4_??.prim"));

  G4String command = "/control/execute ";
  G4String fileName = "";
  if(argc>1)fileName=argv[1];

  if (argc!=1)   // batch mode
    {
      UI->ApplyCommand(command+fileName);    
    }
  else           // interactive mode : define visualization UI terminal
    {
      G4UIsession* session = 0;
#if defined(G4UI_USE_TCSH)
      G4cout << " ****************** TCSH *************************" << G4endl;
      session = new G4UIterminal(new G4UItcsh);
#elif defined(G4UI_USE_XM)
      G4cout << " ******************  XM  *************************" << G4endl;
      session = new G4UIXm(argc,argv);
      UI->ApplyCommand("/control/execute visTutor/gui.mac");      
#elif defined(G4UI_USE_WIN32)
      G4cout << " ****************** WIN32 ************************" << G4endl;
      session = new G4UIWin32();
      UI->ApplyCommand("/control/execute visTutor/gui.mac");      
#elif defined(G4UI_USE_QT)
      G4cout << " ******************  QT  *************************" << G4endl;
      session = new G4UIQt(argc,argv);
      UI->ApplyCommand("/control/execute visTutor/gui.mac");      
#else
      session = new G4UIterminal();
#endif
#ifdef G4VIS_USE
      //G4cout << " ****************** /control/execute myrun.mac *************************" << G4endl;
      //UI->ApplyCommand("/control/execute myrun.mac");     
#endif
      session->SessionStart();

      //char answ='y';
      //G4cout << "delete session? (y/n)" << G4endl;
      //G4cin >> answ;
      //if(answ=='y');

      delete session;

    }

  // Job termination
  // Free the store: user actions, physics_list and detector_description are
  //                 owned and deleted by the run manager, so they should not
  //                 be deleted in the main() program !

  //m_hh->finalize();

  //G4cout <<"VERBOSITY "<<event_action->GetSBVerbosity()<<G4endl;

  //G4cout << Form("cp SB_simul_v0.root %s.root",fileName.c_str()) << G4endl;
  //dum=system(Form("cp SB_simul_v0.root %s.root",fileName.c_str()));

  //G4cout << Form("cp nufluxes_GLOBESformat.txt %s_GLOBESformat.txt",fileName.c_str()) << G4endl;
  //dum=system(Form("cp nufluxes_GLOBESformat.txt %s_GLOBESformat.txt",fileName.c_str()));


#ifdef G4VIS_USE
  delete visManager;
#endif
  //delete histo;
  delete runManager;

  return 0;
}
