//#include "globals.hh"

#include "SBDetectorConstruction.hh"
#include "SBDetectorMessenger.hh"
//#include "SBFieldSetup.hh"
#include "SBField.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Polycone.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4UserLimits.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4Transform3D.hh"
#include "G4RotationMatrix.hh"
#include "G4PVPlacement.hh"
#include "G4PVParameterised.hh"

#include "G4PropagatorInField.hh"
#include "G4ClassicalRK4.hh"
#include "SBField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
#include "G4Mag_UsualEqRhs.hh"
#include "G4MagIntegratorStepper.hh"
#include "G4ChordFinder.hh"

#include "G4Mag_UsualEqRhs.hh"

#include "TMath.h"

#include "Riostream.h"
#include "Rtypes.h"
#include "TROOT.h"
#include "TLine.h"
#include "TVirtualPad.h"
#include "TClass.h"
#include "TVirtualX.h"

#include "G4RunManager.hh"

//to make it a ROOT class
//ClassImp(SBDetectorConstruction)

SBDetectorConstruction::SBDetectorConstruction()
  :TargetMaterial(0),AbsorberMaterial(0),defaultMaterial(0),solidWorld(0),logicWorld(0),physiWorld(0) 
{
  // default parameter values
  TargetThickness=78.*cm;
  TargetDiameter=1.5*cm;

  AbsorberThickness=48.*cm;
  AbsorberDiameter=3.2*cm;

  TunnelLength=40*m;
  TunnelRadius=2*m;
  
  // materials
  DefineMaterials();
  SetTargetMaterial("myBerillium");
  SetAbsorberMaterial("myGraphite");

  // create commands for interactive definition
  detectorMessenger=new SBDetectorMessenger(this);
  //fEmFieldSetup=new SBFieldSetup();
}
//------------------------------
SBDetectorConstruction::~SBDetectorConstruction()
{
  delete detectorMessenger;
  //if (fEmFieldSetup) delete fEmFieldSetup ;
}
//------------------------------
G4VPhysicalVolume* SBDetectorConstruction::Construct()
{
  return ConstructSystem();
}
//------------------------------
G4VPhysicalVolume* SBDetectorConstruction::ConstructSystem()
{
  //
  // Steer the creation of physical volumes
  //  

  bool PSSETUP=false;

  MINIBOONEHORN=false;
  MINOSHORN=false;
  LONGHORN=true;

  //if(HornDesign==0)return 0;
  if(HornDesign==2)MINIBOONEHORN=true;
  if(HornDesign==3)MINOSHORN=true;

  bool placeInnerConductor=true;
  bool placeBfield1=true;
  bool placeClosingRing=true;
  bool placeEndFlange=true;
  bool placeOuterConductor=true;
  bool placeOutCondLnk=true;
  bool placeRefl=true;
  bool placeBfield2=true;
  bool placeReflTopCylinder=true;
  bool placeReflForwardRing=true;
  bool placeTarget=true;
  bool placeTunnel=true;
  bool placeAbsorber=true;

  if(MINIBOONEHORN){
    placeBfield1=true;
    placeClosingRing=false;
    placeEndFlange=false;
    placeOutCondLnk=false;
    if(DoSkin==1){
      placeRefl=true;
      placeInnerConductor=true;
      placeOuterConductor=true;
    }else if(DoSkin==0){
      placeRefl=false;
      placeInnerConductor=false;
      placeOuterConductor=false;
    }
    placeBfield2=true;
    placeReflTopCylinder=false;
    placeReflForwardRing=false;
    placeTarget=true;
    placeTunnel=true;
  }

  if(MINOSHORN){
    placeBfield1=true;
    placeClosingRing=false;
    placeEndFlange=false;
    placeOutCondLnk=false;
    if(DoSkin==1){
      placeRefl=true;
      placeInnerConductor=true;
      placeOuterConductor=true;
    }else if(DoSkin==0){
      placeRefl=false;
      placeInnerConductor=false;
      placeOuterConductor=false;
    }
    placeBfield2=true;
    placeReflTopCylinder=false;
    placeReflForwardRing=false;
    placeTarget=true;
    placeTunnel=true;
    placeAbsorber=true;
  }

  // Clean old geometry, if any
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  SetHornParameters();

  //G4cout<<"pernice "<<DoRefl<<G4endl;
  if(DoRefl==0){
    placeRefl=false;
    placeBfield2=false;
  } else if (DoRefl==1) {
    placeRefl=true;
    placeBfield2=true;
  } else {
    G4cout <<"DoRefl must be either 0 or 1 !!!!"<<endl;
    placeRefl=false;
    placeBfield2=false;
  }

  if(PSSETUP){
    //rO_Hall=2.85*m;
    //L_Hall=55.*m;
    rO_Hall=10.*m;
    L_Hall=1000.*m;
  }

  solidWorld=new G4Tubs("World",0.*cm,rO_Hall,L_Hall/2.,0.*deg,360.*deg);

  logicWorld=new G4LogicalVolume(solidWorld,defaultMaterial,"World");
  physiWorld=new G4PVPlacement(0,G4ThreeVector(),logicWorld,"World",0,false,0);
  G4cout << physiWorld->GetName() <<" created."<<G4endl;

  G4double maxstep=0.1*mm;
  logicWorld->SetUserLimits(new G4UserLimits(maxstep));

  G4cout << "pippo "<< HornDesign <<G4endl;

  bool TargetONLY = false;

  if(!TargetONLY){
    
    if(HornDesign!=0){
      if(!MINOSHORN){ 
	if(!MINIBOONEHORN){
	  DoInnerConductor(placeInnerConductor);
	  DoClosingRing(placeClosingRing);
	  DoInnerConductorEndFlange(placeEndFlange);
	  DoOuterConductor(placeOuterConductor);
	  DoOutCondLnk(placeOutCondLnk);
	  DoReflector(placeRefl);
	  DoReflTopCylinder(placeReflTopCylinder);
	  DoReflForwRing(placeReflForwardRing);
	}
	
	DoMagField1(placeBfield1);
	if(MINIBOONEHORN){
	  DoInnerConductor_MB1(placeInnerConductor);
	  DoOuterConductor_MB1(placeInnerConductor);
	}
	DoMagField2(placeBfield2);
	if(MINIBOONEHORN){
	  DoInnerConductor_MB2(placeRefl);
	  DoOuterConductor_MB2(placeRefl);
	}

      } else if (MINOSHORN){
	
	//ReadHornPar_MS(1,"NOVA_horn_1.txt");
	//ReadHornPar_MS(2,"NOVA_horn_2.txt");
	G4cout << "SBDetectorConstruction input file for horn1: "<<HornFile1<<G4endl;
	G4cout << "SBDetectorConstruction input file for horn2: "<<HornFile2<<G4endl;
	
	ReadHornPar_MS(1,HornFile1);
	ReadHornPar_MS(2,HornFile2);
	
	DoMagField_MS1(placeBfield1);
	DoInnerConductor_MS1(placeInnerConductor);
	DoOuterConductor_MS1(placeInnerConductor);      
	DoMagField_MS2(placeBfield2);      
	DoInnerConductor_MS2(placeRefl);
	DoOuterConductor_MS2(placeRefl);

	DoAbsorber(placeAbsorber);
      }
      
      DoTarget(placeTarget);

      if(PSSETUP){
	DoTunnelPS(placeTunnel);
	DoNear(true);
	DoFar(true);
	DoNearSM(true);
	DoFarSM(true);
      }else{
	DoTunnel(placeTunnel);
      }

      PrintHornParameters();    
      SetVolumesVisibility();
    }
    
  } else { // target only
    
    DoTarget(placeTarget);
    //DoTunnel(placeTunnel);

  }

  return physiWorld;
}

//------------------------------

void SBDetectorConstruction::PrintHornParameters()
{
  G4cout << "==================================================="<< G4endl;
  G4cout << "==================================================="<< G4endl;
  G4cout << "==================================================="<< G4endl;
  G4cout << "HornDesign is "<< GetHornDesign() <<" "/*<<HornDesign*/<< G4endl;
  if(LONGHORN){
    G4cout << "long horn+reflector configuration "<< G4endl; 
  }else{
    G4cout << "short horn+reflector configuration "<< G4endl; 
  }
  G4cout << "Number of horns is "<< GetHornNumber() <<" "/*<<Horn_N*/<< G4endl;
  G4cout << "\n------------------------------------------------------------"
         << "\n---> The target is "
         << TargetThickness/mm << "mm of " << TargetMaterial->GetName() 
         << "\n------------------------------------------------------------\n";
  G4cout << "\n------------------------------------------------------------"
         << "\n---> The absorber is "
         << AbsorberThickness/mm << "mm of " << AbsorberMaterial->GetName() 
         << "\n------------------------------------------------------------\n";
  G4cout<<"*-*-*-*-*-*-*-*-*-*-*-*-*-"<<G4endl;
  G4cout<<"dump all the geom variable"<<G4endl;
  G4cout<<"*-*-*-*-*-*-*-*-*-*-*-*-*-"<<G4endl;
  G4cout<<"InCondCurvRad "<<InCondCurvRad/cm<<" cm "<<G4endl;
  G4cout<<"InCondRad(2) "<<InCondRad[0]/cm<<" "<<InCondRad[1]/cm<<" cm "<<G4endl;
  G4cout<<"InCondDepth(3) "<<InCondDepth[0]/cm<<" "<<InCondDepth[1]/cm<<" "<<InCondDepth[2]/cm<<" cm "<<G4endl;
  G4cout<<"InCondLength(5) "<<InCondLength[0]/cm<<" "<<InCondLength[1]/cm<<" "<<InCondLength[2]/cm<<" "<<InCondLength[3]/cm<<" "<<InCondLength[4]/cm<<" cm "<<G4endl;
  G4cout<<"theta InCondHalfLength "<<theta<<" rad "<<InCondHalfLength/cm<<" cm "<<G4endl;
  G4cout<<"OutCondRad,OutCondDepth,OutCondLength "<<OutCondRad/cm<<" "<<OutCondDepth/cm<<" "<<OutCondLength/cm<<" cm "<<G4endl;
  G4cout<<"OutCond_zpos,ClosRgDepth "<<OutCond_zpos/cm<<" "<<ClosRgDepth/cm<<" cm "<<G4endl;
  G4cout<<"ClosRg_zpos,ClosRg_InRad,ClosRgFoil_zpos "<<ClosRg_zpos/cm<<" "<<ClosRg_InRad/cm<<" "<<ClosRgFoil_zpos/cm<<" cm "<<G4endl;
  G4cout<<"EndFlangeDepth,EndFlangeInRad,EndFlangeOutRad "<<EndFlangeDepth/cm<<" "<<EndFlangeInRad/cm<<" "<<EndFlangeOutRad/cm<<" cm "<<G4endl;
  G4cout<<"EndFlangeTotalDepth,EndFlange_zpos "<<EndFlangeTotalDepth/cm<<" "<<EndFlange_zpos/cm<<" cm "<<G4endl;
  G4cout<<"WaterThickness,SkinThickness "<<WaterThickness/cm<<" "<<SkinThickness/cm<<" cm "<<G4endl;
  G4cout<<"WaterClosingRing_zpos,SkinClosingRing_zpos "<<WaterClosingRing_zpos/cm<<" "<<SkinClosingRing_zpos/cm<<" cm "<<G4endl;
  G4cout<<"OutCondLnkRgLength(3) "<<OutCondLnkRgLength[0]/cm<<" "<<OutCondLnkRgLength[1]/cm<<" "<<OutCondLnkRgLength[2]/cm<<" "<<" cm "<<G4endl;
  G4cout<<"OutCondLnkRgRad(2) "<<OutCondLnkRgRad[0]/cm<<" "<<OutCondLnkRgRad[1]/cm<<G4endl;
  G4cout<<"OutCondLnkTotLength,OutCondLnk_zpos,OutCondLnkRgDepth "<<OutCondLnkTotLength/cm<<" "<<OutCondLnk_zpos/cm<<" "<<OutCondLnkRgDepth/cm<<" cm "<<G4endl;
  G4cout<<"ReflInForwardRad,ReflOutRad,ReflInLength "<<ReflInForwardRad/cm<<" "<<ReflOutRad/cm<<" "<<ReflInLength/cm<<" cm "<<G4endl;
  G4cout<<"Refl_zpos,ReflClRing_zpos,ReflThickness "<<Refl_zpos/cm<<" "<<ReflClRing_zpos/cm<<" "<<ReflThickness/cm<<" cm"<<G4endl;
  G4cout<<"TargetRadius,TargetThickness,TargetZetaPos "<<TargetRadius/cm<<" "<<TargetThickness/cm<<" "<<TargetZetaPos/cm<<" cm "<<G4endl;
  G4cout<<"AbsorberRadius,AbsorberThickness,AbsorberZetaPos "<<AbsorberRadius/cm<<" "<<AbsorberThickness/cm<<" "<<AbsorberZetaPos/cm<<" cm "<<G4endl;
  G4cout<<"HalfMotherVoluLength,Horn_zpos "<<HalfMotherVoluLength/cm<<" "<<Horn_zpos/cm<<" cm "<<G4endl;
  G4cout<<"TunnelRadius,TunnelLength,Tunnel_zpos "<<TunnelRadius/cm<<" "<<TunnelLength/cm<<" "<<Tunnel_zpos/cm<<" cm "<<G4endl;

  G4cout<<"Tunnel starts at z = "<<(Tunnel_zpos-0.5*TunnelLength)/m<<" m "<<G4endl;
  G4cout<<"Target starts at z = "<<(TargetZetaPos-0.5*TargetThickness)/m<<" m "<<G4endl;
  G4cout<<"Absorber starts at z = "<<(AbsorberZetaPos-0.5*AbsorberThickness)/m<<" m "<<G4endl;
  G4cout<<"Horn 1 starts at z = "<<(Horn1ZetaStart)/m<<" m "<<G4endl;
  G4cout<<"Horn 1 ends at z = "<<(Horn1ZetaEnd)/m<<" m "<<G4endl;
  G4cout<<"Horn 2 starts at z = "<<(Horn2ZetaStart)/m<<" m "<<G4endl;
  G4cout<<"Horn 2 ends at z = "<<(Horn2ZetaEnd)/m<<" m "<<G4endl;

  G4cout<<"Horn 1 length = "<<(Horn1ZetaEnd-Horn1ZetaStart)/m<<" m "<<G4endl;
  G4cout<<"Horn 2 length = "<<(Horn2ZetaEnd-Horn2ZetaStart)/m<<" m "<<G4endl;

  G4cout<<"Distance btw target start and tunnel entrance "<<
    ((Tunnel_zpos-0.5*TunnelLength)-(TargetZetaPos-0.5*TargetThickness))/m<<" (m) "<<G4endl;
  G4cout<<"Distance btw target start and horn1 entrance "<<
    ((Horn1ZetaStart)-(TargetZetaPos-0.5*TargetThickness))/m<<" (m) "<<G4endl;
  G4cout<<"Distance btw target start and horn2 entrance "<<
    ((Horn2ZetaStart)-(TargetZetaPos-0.5*TargetThickness))/m<<" (m) "<<G4endl;
  G4cout<<"Distance btw horn1 end and horn2 start "<<
    (Horn2ZetaStart-Horn1ZetaStart)/m<<" (m) "<<G4endl;
  G4cout<<"Distance btw horn2 end and tunnel entrance "<<
    ((Tunnel_zpos-0.5*TunnelLength)-Horn2ZetaEnd)/m<<" (m) "<<G4endl;

  G4cout<<"*-*-*-*-*-*-*-*-*-*-*-*-*-"<<G4endl;
  G4cout<<" Inner conductor : "<<G4endl;
  G4cout<<"theta="<<theta<<G4endl;
  G4cout<<"0 position z= "<<zPlane[0]<<" rI= "<<rInner[0]<<" rO = "<<rOuter[0]<<G4endl;
  G4cout<<"1 position z= "<<zPlane[1]<<" rI= "<<rInner[1]<<" rO = "<<rOuter[1]<<G4endl;
  G4cout<<"2 position z= "<<zPlane[2]<<" rI= "<<rInner[2]<<" rO = "<<rOuter[2]<<G4endl;
  G4cout<<"3 position z= "<<zPlane[3]<<" rI= "<<rInner[3]<<" rO = "<<rOuter[3]<<G4endl;

  G4cout <<"Horn X: "<< Horn_xpos/cm <<" Y: "<<Horn_ypos/cm<<" (cm)."<< G4endl;

  G4cout<<"Min. allowed Horn displacement radius: "<<MinHornDisplacRad/cm<<" cm."<<G4endl;
  G4cout<<"Use conductor skin : "<<DoSkin<<" (0=no,1=yes)"<<G4endl;
  G4cout<<"Use reflector : "<<DoRefl<<" (0=no,1=yes)"<<G4endl;
  G4cout<<"Injection schema : "<<InjSchema<<" (0=one by one, 1=all at once)"<<G4endl;
  G4cout<<"Off Axis Angle : "<<OffAxisAngle/rad<<" (rad)"<<G4endl;
  G4cout<<"Off Axis phi Angle : "<<OffAxisPhi/rad<<" (rad)"<<G4endl;
  G4cout << "==================================================="<< G4endl;
  G4cout << "==================================================="<< G4endl;
  G4cout << "==================================================="<< G4endl;

  G4cout<<"HORNSHAPE_TARG "<< (TargetZetaPos-0.5*TargetThickness-InCond_zpos-Horn_zpos)/mm<<" "<<TargetThickness/mm<<" "<<TargetDiameter/mm<<G4endl; 

}

//------------------------------
void SBDetectorConstruction::SetTargetMaterial(G4String materialChoice)
{
  G4Material* pttoMaterial=G4Material::GetMaterial(materialChoice);     
  if (pttoMaterial) TargetMaterial=pttoMaterial;
}
//------------------------------
void SBDetectorConstruction::SetAbsorberMaterial(G4String materialChoice)
{
  G4Material* pttoMaterial=G4Material::GetMaterial(materialChoice);     
  if (pttoMaterial) AbsorberMaterial=pttoMaterial;
}
//------------------------------
void SBDetectorConstruction::SetOffAxisAngle(G4double val){OffAxisAngle=val;}
void SBDetectorConstruction::SetOffAxisPhi(G4double val){OffAxisPhi=val;}
//------------------------------
void SBDetectorConstruction::SetTargetThickness(G4double val){TargetThickness=val;}
//------------------------------
void SBDetectorConstruction::SetAbsorberThickness(G4double val){AbsorberThickness=val;}
//------------------------------
void SBDetectorConstruction::SetHornNumber(G4int val){Horn_N=val;}
void SBDetectorConstruction::SetHornX(G4double val){Horn_xpos=val;}
void SBDetectorConstruction::SetHornY(G4double val){Horn_ypos=val;}
//------------------------------
void SBDetectorConstruction::SetTargetDiameter(G4double val){TargetDiameter=val;}
//------------------------------
void SBDetectorConstruction::SetAbsorberDiameter(G4double val){AbsorberDiameter=val;}
//------------------------------
void SBDetectorConstruction::SetHornDesign(G4int val){HornDesign=val;}
void SBDetectorConstruction::SetInj(G4int val){InjSchema=val;}
void SBDetectorConstruction::SetSkin(G4int val){DoSkin=val;}
void SBDetectorConstruction::SetRefl(G4int val){DoRefl=val;}

void SBDetectorConstruction::SetHorn_L1(G4double val){MB_L1=val;}
void SBDetectorConstruction::SetHorn_L2(G4double val){MB_L2=val;}
void SBDetectorConstruction::SetHorn_L3(G4double val){MB_L3=val;}
void SBDetectorConstruction::SetHorn_L4(G4double val){MB_L4=val;}
void SBDetectorConstruction::SetHorn_L5(G4double val){MB_L5=val;}
void SBDetectorConstruction::SetHorn_R(G4double val){MB_R=val;}
void SBDetectorConstruction::SetHorn_r(G4double val){MB_r=val;}
void SBDetectorConstruction::SetHorn_R0(G4double val){MB_R0=val;}
void SBDetectorConstruction::SetHorn_R1(G4double val){MB_R1=val;}
void SBDetectorConstruction::SetHorn_R2(G4double val){MB_R2=val;}
//------------------------------
void SBDetectorConstruction::SetHorn_I1(G4double val){Horn_I1=val;}
void SBDetectorConstruction::SetHorn_I2(G4double val){Horn_I2=val;}
//------------------------------
void SBDetectorConstruction::SetHorn2_L1(G4double val){MB2_L1=val;}
void SBDetectorConstruction::SetHorn2_L2(G4double val){MB2_L2=val;}
void SBDetectorConstruction::SetHorn2_L3(G4double val){MB2_L3=val;}
void SBDetectorConstruction::SetHorn2_L4(G4double val){MB2_L4=val;}
void SBDetectorConstruction::SetHorn2_L5(G4double val){MB2_L5=val;}
void SBDetectorConstruction::SetHorn2_R(G4double val){MB2_R=val;}
void SBDetectorConstruction::SetHorn2_r(G4double val){MB2_r=val;}
void SBDetectorConstruction::SetHorn2_R0(G4double val){MB2_R0=val;}
void SBDetectorConstruction::SetHorn2_R1(G4double val){MB2_R1=val;}
void SBDetectorConstruction::SetHorn2_R2(G4double val){MB2_R2=val;}
void SBDetectorConstruction::SetHorn2_DZ(G4double val){MB2_DZ=val;}
//------------------------------
void SBDetectorConstruction::SetTarget_DZ(G4double val){Target_DZ=val;}
void SBDetectorConstruction::SetAbsorber_DZ(G4double val){Absorber_DZ=val;}
void SBDetectorConstruction::SetHornFile1(G4String val){HornFile1=val;}
void SBDetectorConstruction::SetHornFile2(G4String val){HornFile2=val;}
//------------------------------
//G4int SBDetectorConstruction::GetHornDesign(){return HornDesign;}
G4double SBDetectorConstruction::GetHorn_L1(){return MB_L1;}
G4double SBDetectorConstruction::GetHorn_L2(){return MB_L2;}
G4double SBDetectorConstruction::GetHorn_L3(){return MB_L3;}
G4double SBDetectorConstruction::GetHorn_L4(){return MB_L4;}
G4double SBDetectorConstruction::GetHorn_L5(){return MB_L5;}
G4double SBDetectorConstruction::GetHorn_R(){return MB_R;}
G4double SBDetectorConstruction::GetHorn_r(){return MB_r;}
G4double SBDetectorConstruction::GetHorn_R0(){return MB_R0;}
G4double SBDetectorConstruction::GetHorn_R1(){return MB_R1;}
G4double SBDetectorConstruction::GetHorn_R2(){return MB_R2;}
//------------------------------
G4double SBDetectorConstruction::GetHorn_I1(){return Horn_I1;}
G4double SBDetectorConstruction::GetHorn_I2(){return Horn_I2;}
//------------------------------
G4double SBDetectorConstruction::GetHorn2_L1(){return MB2_L1;}
G4double SBDetectorConstruction::GetHorn2_L2(){return MB2_L2;}
G4double SBDetectorConstruction::GetHorn2_L3(){return MB2_L3;}
G4double SBDetectorConstruction::GetHorn2_L4(){return MB2_L4;}
G4double SBDetectorConstruction::GetHorn2_L5(){return MB2_L5;}
G4double SBDetectorConstruction::GetHorn2_R(){return MB2_R;}
G4double SBDetectorConstruction::GetHorn2_r(){return MB2_r;}
G4double SBDetectorConstruction::GetHorn2_R0(){return MB2_R0;}
G4double SBDetectorConstruction::GetHorn2_R1(){return MB2_R1;}
G4double SBDetectorConstruction::GetHorn2_R2(){return MB2_R2;}
G4double SBDetectorConstruction::GetHorn2_DZ(){return MB2_DZ;}
//------------------------------
G4double SBDetectorConstruction::GetTarget_DZ(){return Target_DZ;}
G4double SBDetectorConstruction::GetAbsorber_DZ(){return Absorber_DZ;}
G4double SBDetectorConstruction::GetHornX(){return Horn_xpos;}
G4double SBDetectorConstruction::GetHornY(){return Horn_ypos;}
//------------------------------
void SBDetectorConstruction::UpdateGeometry()
{
  G4cout << "SBDetectorConstruction::UpdateGeometry "<< G4endl;  
  G4RunManager::GetRunManager()->DefineWorldVolume(ConstructSystem());
}
//------------------------------
void SBDetectorConstruction::SetHornParameters()
{
  G4cout << "SBDetectorConstruction::SetHornParameters "<< G4endl;  
  current1=GetHorn_I1();//= 300000.*ampere;
  current2=GetHorn_I2();//= 600000.*ampere;

  InCond_zpos=MB2_DZ;

  InCondCurvRad=2.0*cm;
  InCondRad[0]=3.7*cm;
  InCondRad[1]=15.4*cm;

  InCondDepth[0]=0.3*cm;
  InCondDepth[1]=0.3*cm;
  InCondDepth[2]=0.3*cm;

  InCondLength[0]=2.*cm;
  InCondLength[1]=0.5*cm;
  InCondLength[2]=36.6*cm;

  if(LONGHORN){
    InCondLength[3]=98.2*cm;//LONGHORN_ANDREA
  }else{
    InCondLength[3]=78.2*cm;
  }

  InCondLength[4]=1.8*cm;

  InCondHalfLength=0.5*(InCondLength[0]+InCondLength[1]+InCondLength[2]+InCondLength[3]+InCondLength[4]);
  
  // Outer Conductor (tube)  
  OutCondRad=20.*cm;
  OutCondDepth=0.3*cm;

  if(LONGHORN){
    OutCondLength=129.9*cm;//LONGHORN_ANDREA
    //OutCondLength=131.3*cm;//LONGHORN_ANDREA +1.4cm+2.3cm modified to match with the outer conductor link
  }else{
    OutCondLength=109.9*cm;
  }

  OutCond_zpos=InCondHalfLength-0.5*OutCondLength;

  // Inner Conductor Closing Ring (tube + circular foil)    
  ClosRgDepth=0.3*cm;

  //Inner Condutor End Flange
  EndFlangeDepth=2.65*cm;
  EndFlangeInRad=8.1*cm;
  EndFlangeOutRad=30.5*cm;

  // Double Skin (Water surround the horn and the double skin surround the water)
  WaterThickness=0.2*cm;
  SkinThickness=0.2*cm;

  //Outer conductor Link Ring and Outer Conductor End flange
  OutCondLnkRgLength[0]=2.2*cm;
  OutCondLnkRgLength[1]=3.3*cm;
  OutCondLnkRgLength[2]=2.3*cm;
  OutCondLnkRgDepth=1.05*cm;
  OutCondLnkRgRad[0]=16.475*cm;

  OutCondLnkRgRad[1]=44.75*cm;

  OutCondLnkTotLength=OutCondLnkRgLength[0]+OutCondLnkRgLength[1]+OutCondLnkRgLength[2];
  OutCondLnk_zpos=OutCond_zpos-0.5*(OutCondLength+OutCondLnkTotLength);

  // Reflector
  ReflInForwardRad=35.*cm;
  ReflOutRad=39.7*cm;

  if(LONGHORN){
    //ReflInLength=211.*cm;//LONGHORN_ANDREA
    ReflInLength=226.*cm;//as in g3 modified by me
  }else{
    ReflInLength=196.*cm;
  }

  ReflThickness=0.3*cm;

  // Target
  TargetRadius=TargetDiameter/2.;
  // Absorber
  AbsorberRadius=AbsorberDiameter/2.;

  // variables for MINIBOONE HORN
  MBcurve1_n=11;
  MBcurve2_n=11;
  MBzplanes=MBcurve1_n + MBcurve2_n + 6;
  
  NMB=28;

  MB_L1=GetHorn_L1();
  MB_L2=GetHorn_L2();
  MB_L3=GetHorn_L3();
  MB_L4=GetHorn_L4();
  MB_L5=GetHorn_L5();
  MB_R=GetHorn_R();
  MB_r=GetHorn_r();
  
  MB_R0=GetHorn_R0();
  MB_R1=GetHorn_R1();
  MB_R2=GetHorn_R2();

  MB_totL=MB_r+MB_L1+MB_L2+MB_L3+MB_L4+MB_L5+MB_R+MB1_SkinThickness;

  MB2_L1=GetHorn2_L1();
  MB2_L2=GetHorn2_L2();
  MB2_L3=GetHorn2_L3();
  MB2_L4=GetHorn2_L4();
  MB2_L5=GetHorn2_L5();
  MB2_R=GetHorn2_R();
  MB2_r=GetHorn2_r();
  
  MB2_R0=GetHorn2_R0();
  MB2_R1=GetHorn2_R1();
  MB2_R2=GetHorn2_R2();

  MB2_DZ=GetHorn2_DZ();

  MB2_totL=MB2_r+MB2_L1+MB2_L2+MB2_L3+MB2_L4+MB2_L5+MB2_R+MB2_SkinThickness;

  MinHornDisplacRad=(sqrt(2.)*MB2_R1)+2*cm;
  if(Horn_N==1)MinHornDisplacRad=-1;

  // the space for the Horn (~mother)
  HalfMotherVoluLength=0;

  Clearance=3.*m;
  HalfHornLength=0;

  if(!MINIBOONEHORN)Clearance=0.*m;

  HalfHornLength=InCondHalfLength-OutCondLength+ReflInLength+ReflThickness;
  if(MINIBOONEHORN)HalfHornLength=MB_totL/2.;
  if(MINIBOONEHORN)HalfHornLength=TMath::Max(MB_totL/2.,MB2_totL/2.);

  //if(MINOSHORN)HalfHornLength=130.*m;
  if(MINOSHORN)HalfHornLength=250.*m;
  //if(MINOSHORN)HalfHornLength=5.*m;//PS

  HalfMotherVoluLength=HalfHornLength+Clearance/2.;
  
  rO_Hall=ReflOutRad+ReflThickness+1.*cm;
  if(rO_Hall<TunnelRadius)rO_Hall=TunnelRadius;
  L_Hall=TunnelLength+2.*HalfMotherVoluLength;

  //G4double Horn_zpos=-0.5*TunnelLength;// original
  //G4double Horn_zpos=-0.5*L_Hall+BackClearance;
  Horn_xpos=GetHornX();
  Horn_ypos=GetHornY();
  Horn_zpos=-0.5*L_Hall+2.*HalfMotherVoluLength-HalfHornLength;
  //if(MINIBOONEHORN)Horn_zpos=-0.5*L_Hall+2*HalfMotherVoluLength-MB_totL;

}

//----------------------------------------------
void SBDetectorConstruction::DoInnerConductorEndFlange(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductorEndFlange "<< G4endl;

  G4double phi0=0.,phi1=360.*deg;

  G4double zPl_EF[4]={0};
  G4double rI_EF[4]={0};
  G4double rO_EF[4]={0};
  
  EndFlangeTotalDepth=EndFlangeDepth+InCondDepth[0];

  rI_EF[0]=EndFlangeInRad;
  rI_EF[1]=EndFlangeInRad;
  rI_EF[2]=InCondRad[0];
  rI_EF[3]=InCondRad[0];

  zPl_EF[0]=-0.5*EndFlangeTotalDepth;
  zPl_EF[1]=-0.5*EndFlangeTotalDepth+EndFlangeDepth;
  zPl_EF[2]=-0.5*EndFlangeTotalDepth+EndFlangeDepth;
  zPl_EF[3]=0.5*EndFlangeTotalDepth;

  rO_EF[0]=EndFlangeOutRad;
  rO_EF[1]=EndFlangeOutRad;
  rO_EF[2]=EndFlangeOutRad;
  rO_EF[3]=EndFlangeOutRad;

  //for(int i=0;i<4;i++){
  //G4cout<<i<<" position z= "<<zPl_EF[i]<<" rI_EF= "<<rI_EF[i]<<" rO_EF="<<rO_EF[i]<<G4endl;
  //}

  EndFlange_zpos=-InCondHalfLength-0.5*EndFlangeTotalDepth;

  solidEndFlange=new G4Polycone("ICEF",phi0,phi1,4,zPl_EF,rI_EF,rO_EF);
  logicEndFlange=new G4LogicalVolume(solidEndFlange,HornMaterial,"ICEF");
  if(place){
    physiEndFlange=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,EndFlange_zpos+Horn_zpos),logicEndFlange,"ICEF",logicWorld,false,0);
    G4cout << physiEndFlange->GetName() <<" created."<<G4endl;
  }

}

//----------------------------------------------
void SBDetectorConstruction::DoOuterConductor(bool place)
{
  G4cout << "SBDetectorConstruction::DoOuterConductor "<< G4endl;
  solidOuterConductor=new G4Tubs("OUTC",OutCondRad,OutCondRad+OutCondDepth,OutCondLength/2.,0.*deg,360.*deg);
  logicOuterConductor=new G4LogicalVolume(solidOuterConductor,HornMaterial,"OUTC");
  if(place)
    physiOuterConductor=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,OutCond_zpos+Horn_zpos),logicOuterConductor,"OUTC",logicWorld,false,0); 
  if(place)G4cout << physiOuterConductor->GetName() <<" created."<<G4endl;
}
//----------------------------------------------
void SBDetectorConstruction::DoOutCondLnk(bool place)
{
  G4cout << "SBDetectorConstruction::DoOutCondLnk "<< G4endl;  
  G4double zPl_OL[6]={0};
  G4double rI_OL[6]={0};
  G4double rO_OL[6]={0};

  zPl_OL[0]=-0.5*OutCondLnkTotLength;
  zPl_OL[1]=-0.5*OutCondLnkTotLength+OutCondLnkRgLength[0];
  zPl_OL[2]=-0.5*OutCondLnkTotLength+OutCondLnkRgLength[0];
  zPl_OL[3]=-0.5*OutCondLnkTotLength+OutCondLnkRgLength[0]+OutCondLnkRgLength[1];
  zPl_OL[4]=-0.5*OutCondLnkTotLength+OutCondLnkRgLength[0]+OutCondLnkRgLength[1];
  zPl_OL[5]=0.5*OutCondLnkTotLength;

  rI_OL[0]=OutCondLnkRgRad[0];
  rI_OL[1]=OutCondLnkRgRad[0]+OutCondLnkRgLength[0]/sqrt(2.);
  rI_OL[2]=OutCondLnkRgRad[1]-OutCondLnkRgDepth;
  rI_OL[3]=OutCondLnkRgRad[1]-OutCondLnkRgDepth;
  rI_OL[4]=OutCondRad;
  rI_OL[5]=OutCondRad;

  rO_OL[0]=OutCondLnkRgRad[1];
  rO_OL[1]=OutCondLnkRgRad[1];
  rO_OL[2]=OutCondLnkRgRad[1];
  rO_OL[3]=OutCondLnkRgRad[1];
  rO_OL[4]=OutCondLnkRgRad[1];
  rO_OL[5]=OutCondLnkRgRad[1];

  solidOutCondLnk=new G4Polycone("OCLR",0*deg,360*deg,6,zPl_OL,rI_OL,rO_OL);
  logicOutCondLnk=new G4LogicalVolume(solidOutCondLnk,HornMaterial,"OCLR");
  if(place){
    physiOutCondLnk=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,OutCondLnk_zpos+Horn_zpos),logicOutCondLnk,"OCLR",logicWorld,false,0);
    G4cout << physiOutCondLnk->GetName() <<" created."<<G4endl;
  }
}

//----------------------------------------------
void SBDetectorConstruction::DoMagField1(bool place)
{
  G4cout << "SBDetectorConstruction::DoMagField1 "<< G4endl;  
  for(int i=0;i<8;i++){
    rInnerB[i]=0;
    rOuterB[i]=0;
    zPlaneB[i]=0;
    zPlaneB[i]=zPlane[i];
  }

  // magnetic field region shifterd forward to avoid clash with OCLR
  G4double B_zshift=OutCondLnkRgLength[2];
  zPlaneB[0]=zPlane[0]+B_zshift;
  zPlaneB[1]=zPlaneB[0];// drop initial "ring of 2cm"

  rInnerB[0]=InCondRad[0]+InCondDepth[0];
  rInnerB[1]=InCondRad[0]+InCondDepth[0];
  rInnerB[2]=InCondRad[0]+InCondDepth[1];
  rInnerB[3]=InCondRad[0]+InCondDepth[1];

  rOuterB[0]=OutCondRad-0.1*mm;
  rOuterB[1]=OutCondRad-0.1*mm;
  rOuterB[2]=OutCondRad-0.1*mm;
  rOuterB[3]=OutCondRad-0.1*mm;

  for(int i=0;i<np;i++){
    rInnerB[4+i]=rInner[4+i]+InCondDepth[2];
    rOuterB[4+i]=OutCondRad;
  }

  for(G4int j=0;j<8;j++){
    G4cout << "zPlane "<<j<<" "<<zPlane[j]/cm<<" cm "<<rInner[j]/cm<<" cm "<<rOuter[j]/cm<<" cm "<<G4endl;
    G4cout << "zPlaneB "<<j<<" "<<zPlaneB[j]/cm<<" cm "<<rInnerB[j]/cm<<" cm "<<rOuterB[j]/cm<<" cm "<<G4endl;
  }

  G4double stepMinimum1=1.0e-3*mm;
  SBField* myField1[4];
  G4Mag_UsualEqRhs* iEquation1[4];
  G4MagIntegratorStepper* iStepper1[4];
  G4ChordFinder* iChordFinder1[4];
  G4FieldManager* mfieldMgr1[4];

  for(int i=0;i<Horn_N;i++){
    myField1[i]=new SBField(current1,i,this);
    iEquation1[i]=new G4Mag_UsualEqRhs(myField1[i]);
    iStepper1[i]=new G4ClassicalRK4(iEquation1[i]);
    iChordFinder1[i]=new G4ChordFinder(myField1[i],stepMinimum1,iStepper1[i]);
    mfieldMgr1[i]=new G4FieldManager(myField1[i],iChordFinder1[i]);
  }

  for(int i=0;i<NMB;i++){
    rInnerMB[i]=0;
    rOuterMB[i]=0;
    zPlaneMB[i]=0;
  }
  dalpha1=0.5*M_PI/G4double(MBcurve1_n);
  dalpha2=0.5*M_PI/G4double(MBcurve2_n);
  nn0=MBcurve1_n+6;//17
  nn1=MBcurve2_n;
  
  if(!MINIBOONEHORN){
    solidBField1=new G4Polycone("BFL1",0.*deg,360.*deg,numZPlanes,zPlaneB,rInnerB,rOuterB);
  } else {
    for(int i=0;i<MBzplanes;i++){
      rInnerMB[i]=MB_R0;
      rOuterMB[i]=MB_R1;
    }
    rInnerMB[MBcurve1_n+2]=MB_R2;
    rInnerMB[MBcurve1_n+3]=MB_R2;    
    for(int i=0;i<MBcurve1_n;i++){
      zPlaneMB[i]=-0.5*MB_totL+MB_r*(1-cos(i*dalpha1));
      rInnerMB[i]=rInnerMB[i] + MB_r*(1-sin(i*dalpha1));
      rOuterMB[i]=rOuterMB[i] - MB_r*(1-sin(i*dalpha1));
      //G4cout <<" check I "<< i <<" "<< rInnerMB[i]/cm <<" "<< zPlaneMB[i]/cm << G4endl;
      //G4cout <<" check O "<< i <<" "<< rOuterMB[i]/cm << G4endl;
    }
    zPlaneMB[MBcurve1_n]=zPlaneMB[0]+MB_r;//11
    zPlaneMB[MBcurve1_n+1]=zPlaneMB[MBcurve1_n]+MB_L1;//12
    zPlaneMB[MBcurve1_n+2]=zPlaneMB[MBcurve1_n+1]+MB_L2;//13
    zPlaneMB[MBcurve1_n+3]=zPlaneMB[MBcurve1_n+2]+MB_L3;//14
    zPlaneMB[MBcurve1_n+4]=zPlaneMB[MBcurve1_n+3]+MB_L4;//15
    zPlaneMB[MBcurve1_n+5]=zPlaneMB[MBcurve1_n+4]+MB_L5;//16
    for(int i=0;i<nn1;i++){
      zPlaneMB[i+nn0]=zPlaneMB[nn0-1]+MB_R*sin(i*dalpha2);
      rInnerMB[i+nn0]= rInnerMB[i+nn0]+MB_R*(1-cos(i*dalpha2));
      rOuterMB[i+nn0]= rOuterMB[i+nn0]-MB_R*(1-cos(i*dalpha2));
      //G4cout << i+nn0 << G4endl;

// tmp!!! to reproduce nufact style horn for Christoph
      //rInnerMB[i+nn0]= MB_R2+MB_R*(1-cos(i*dalpha2));
// tmp
    }
// tmp!!! to reproduce nufact style horn for Christoph
      //rInnerMB[nn0-1]= MB_R2;
      //rInnerMB[nn0-2]= MB_R2;
// tmp


    solidBField1=new G4Polycone("BFL1",0.*deg,360.*deg,MBzplanes,zPlaneMB,rInnerMB,rOuterMB);
  }

  for(int i=0;i<Horn_N;i++){
    logicBField1[i]=new G4LogicalVolume(solidBField1,FillingMaterial,Form("BFL1_%d",i),mfieldMgr1[i],0,0);
  }

  if(place){
    physiBField1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos+InCond_zpos),logicBField1[0],"BFL1_0",logicWorld,false,0);
    G4cout << physiBField1[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      if(InjSchema==1){//B field in all horns simultaneously
	physiBField1[1]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos+InCond_zpos),logicBField1[1],"BFL1_1",logicWorld,false,0);
	G4cout << physiBField1[1]->GetName() <<"[1] created."<<G4endl;
	physiBField1[2]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos+InCond_zpos),logicBField1[2],"BFL1_2",logicWorld,false,0);
	G4cout << physiBField1[2]->GetName() <<"[2] created."<<G4endl;
	physiBField1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos+InCond_zpos),logicBField1[3],"BFL1_3",logicWorld,false,0);
	G4cout << physiBField1[3]->GetName() <<"[3] created."<<G4endl;
      }
    }else{
      G4cout<< "SBDetectorConstruction::DoMagField1 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }

}


//----------------------------------------------
void SBDetectorConstruction::DoMagField2(bool place)
{
  G4cout << "SBDetectorConstruction::DoMagField2 "<< G4endl;  
  G4double rI_BFL2[3]={0};
  G4double rO_BFL2[3]={0};
  rI_BFL2[0]=OutCondRad+OutCondDepth+ReflThickness;
  rI_BFL2[1]=OutCondRad+OutCondDepth+ReflThickness;
  rI_BFL2[2]=ReflInForwardRad+ReflThickness;
  rO_BFL2[0]=ReflOutRad;
  rO_BFL2[1]=ReflOutRad;
  rO_BFL2[2]=ReflOutRad;

  G4double stepMinimum2=1.0e-3*mm;
  SBField* myField2[4];
  G4Mag_UsualEqRhs* iEquation2[4];
  G4MagIntegratorStepper* iStepper2[4];
  G4ChordFinder* iChordFinder2[4];
  G4FieldManager* mfieldMgr2[4];

  for(int i=0;i<Horn_N;i++){
    myField2[i]=new SBField(current2,i,this);
    iEquation2[i]=new G4Mag_UsualEqRhs(myField2[i]);
    iStepper2[i]=new G4ClassicalRK4(iEquation2[i]);
    iChordFinder2[i]=new G4ChordFinder(myField2[i],stepMinimum2,iStepper2[i]);
    mfieldMgr2[i]=new G4FieldManager(myField2[i],iChordFinder2[i]);
  }

  if(!MINIBOONEHORN){
    solidBField2=new G4Polycone("BFL2",0.*deg,360.*deg,3,zPl_RE,rI_BFL2,rO_BFL2);
  } else {
    for(int i=0;i<MBzplanes;i++){
      rInnerMB[i]=MB2_R0;
      rOuterMB[i]=MB2_R1;
    }
    rInnerMB[MBcurve1_n+2]=MB2_R2;
    rInnerMB[MBcurve1_n+3]=MB2_R2;    
    for(int i=0;i<MBcurve1_n;i++){
      zPlaneMB[i]=-0.5*MB2_totL+MB2_r*(1-cos(i*dalpha1));
      rInnerMB[i]=rInnerMB[i] + MB2_r*(1-sin(i*dalpha1));
      rOuterMB[i]=rOuterMB[i] - MB2_r*(1-sin(i*dalpha1));
    }
    zPlaneMB[MBcurve1_n]=zPlaneMB[0]+MB2_r;//11
    zPlaneMB[MBcurve1_n+1]=zPlaneMB[MBcurve1_n]+MB2_L1;//12
    zPlaneMB[MBcurve1_n+2]=zPlaneMB[MBcurve1_n+1]+MB2_L2;//13
    zPlaneMB[MBcurve1_n+3]=zPlaneMB[MBcurve1_n+2]+MB2_L3;//14
    zPlaneMB[MBcurve1_n+4]=zPlaneMB[MBcurve1_n+3]+MB2_L4;//15
    zPlaneMB[MBcurve1_n+5]=zPlaneMB[MBcurve1_n+4]+MB2_L5;//16
    for(int i=0;i<nn1;i++){
      zPlaneMB[i+nn0]=zPlaneMB[nn0-1]+MB2_R*sin(i*dalpha2);
      rInnerMB[i+nn0]= rInnerMB[i+nn0]+MB2_R*(1-cos(i*dalpha2));
      rOuterMB[i+nn0]= rOuterMB[i+nn0]-MB2_R*(1-cos(i*dalpha2));
    }
    //for(int i=0;i<MBzplanes;i++){
    //G4cout <<"MINIBOONE POLYCONE 2 "<< i <<" x="<< zPlaneMB[i]/cm <<" cm. rI="<< rInnerMB[i]/cm<< " cm. rO="<< rOuterMB[i]/cm << " cm."<< G4endl;
    //}
    solidBField2=new G4Polycone("BFL2",0.*deg,360.*deg,MBzplanes,zPlaneMB,rInnerMB,rOuterMB);
  }
  for(int i=0;i<Horn_N;i++){
    logicBField2[i]=new G4LogicalVolume(solidBField2,FillingMaterial,Form("BFL2_%d",i),mfieldMgr2[i],0,0);
  }

  if(MINIBOONEHORN)Refl_zpos=0;

  if(place){
    physiBField2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos+Refl_zpos),logicBField2[0],"BFL2_0",logicWorld,false,0);
    G4cout << physiBField2[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      if(InjSchema==1){//B field in all horns simultaneously
	physiBField2[1]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos+Refl_zpos),logicBField2[1],"BFL2_1",logicWorld,false,0);
	G4cout << physiBField2[1]->GetName() <<"[1] created."<<G4endl;
	physiBField2[2]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos+Refl_zpos),logicBField2[2],"BFL2_2",logicWorld,false,0);
	G4cout << physiBField2[2]->GetName() <<"[2] created."<<G4endl;
	physiBField2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos+Refl_zpos),logicBField2[3],"BFL2_3",logicWorld,false,0);
	G4cout << physiBField2[3]->GetName() <<"[3] created."<<G4endl;
      }
    }else{
      G4cout<< "SBDetectorConstruction::DoMagField2 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }


}

//----------------------------------------------
void SBDetectorConstruction::SetVolumesVisibility()
{
  G4cout << "SBDetectorConstruction::SetVolumesVisibility "<< G4endl;  
  G4VisAttributes* targetcol= new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  targetcol->SetVisibility(true);
  if(logicTarget)logicTarget->SetVisAttributes(targetcol);

  G4VisAttributes* absorbercol= new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  absorbercol->SetVisibility(true);
  if(logicAbsorber)logicAbsorber->SetVisAttributes(absorbercol);


  G4VisAttributes* InnerConductorcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  InnerConductorcol->SetVisibility(true);
  if(logicInnerConductor)logicInnerConductor->SetVisAttributes(InnerConductorcol);

  G4VisAttributes* OuterConductorcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  OuterConductorcol->SetVisibility(true);
  if(logicOuterConductor)logicOuterConductor->SetVisAttributes(OuterConductorcol);

  G4VisAttributes* OutCondLnkcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  OutCondLnkcol->SetVisibility(true);
  if(logicOutCondLnk)logicOutCondLnk->SetVisAttributes(OutCondLnkcol);

  G4VisAttributes* ClosingRingcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  ClosingRingcol->SetVisibility(true);
  if(logicClosingRing)logicClosingRing->SetVisAttributes(ClosingRingcol);

  G4VisAttributes* EndFlangecol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  EndFlangecol->SetVisibility(true);
  if(logicEndFlange)logicEndFlange->SetVisAttributes(EndFlangecol);

  G4VisAttributes* TopCylindercol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  TopCylindercol->SetVisibility(true);
  if(logicTopCylinder)logicTopCylinder->SetVisAttributes(TopCylindercol);

  G4VisAttributes* ReflForwardRingcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  ReflForwardRingcol->SetVisibility(true);
  if(logicReflForwardRing)logicReflForwardRing->SetVisAttributes(ReflForwardRingcol);

  G4VisAttributes* Reflcol= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  Reflcol->SetVisibility(true);
  if(logicRefl)logicRefl->SetVisAttributes(Reflcol);

  //G4VisAttributes* BField1col= new G4VisAttributes(G4Colour(0.4,0.5,0.2));
  G4VisAttributes* BField1col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  BField1col->SetVisibility(true);
  for(int i=0;i<Horn_N;i++){
    if(logicBField1[i])logicBField1[i]->SetVisAttributes(BField1col);
    if(logicBField_MS1[i])logicBField_MS1[i]->SetVisAttributes(BField1col);
  }
  //G4VisAttributes* BField2col= new G4VisAttributes(G4Colour(0.4,0.5,0.2));
  G4VisAttributes* BField2col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  BField2col->SetVisibility(true);
  for(int i=0;i<Horn_N;i++){
    if(logicBField2[i])logicBField2[i]->SetVisAttributes(BField2col);
    if(logicBField_MS2[i])logicBField_MS2[i]->SetVisAttributes(BField2col);
  }

  G4VisAttributes* ECAPcol= new G4VisAttributes(G4Colour(0.8,0.1,0.3));
  ECAPcol->SetVisibility(true);
  if(logicBECAP1)logicBECAP1->SetVisAttributes(ECAPcol);
  if(logicFECAP1)logicFECAP1->SetVisAttributes(ECAPcol);
  if(logicBECAP2)logicBECAP2->SetVisAttributes(ECAPcol);
  if(logicFECAP2)logicFECAP2->SetVisAttributes(ECAPcol);


  //G4VisAttributes* InnerConductor_MB1col= new G4VisAttributes(G4Colour(0.1,1.0,0.3));
  G4VisAttributes* InnerConductor_MB1col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  InnerConductor_MB1col->SetVisibility(true);
  if(logicInnerConductor_MB1)logicInnerConductor_MB1->SetVisAttributes(InnerConductor_MB1col);

  //G4VisAttributes* InnerConductor_MB2col= new G4VisAttributes(G4Colour(0.1,1.0,0.3));
  G4VisAttributes* InnerConductor_MB2col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  InnerConductor_MB2col->SetVisibility(true);
  if(logicInnerConductor_MB2)logicInnerConductor_MB2->SetVisAttributes(InnerConductor_MB2col);

  G4VisAttributes* OuterConductor_MB1col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  OuterConductor_MB1col->SetVisibility(true);
  if(logicOuterConductor_MB1)logicOuterConductor_MB1->SetVisAttributes(OuterConductor_MB1col);

  G4VisAttributes* OuterConductor_MB2col= new G4VisAttributes(G4Colour(0.5,0.5,0.5));
  OuterConductor_MB2col->SetVisibility(true);
  if(logicOuterConductor_MB2)logicOuterConductor_MB2->SetVisAttributes(OuterConductor_MB2col);


}
//----------------------------------------------
void SBDetectorConstruction::DoClosingRing(bool place)
{
  G4cout << "SBDetectorConstruction::DoClosingRing "<< G4endl;  
  WaterClosingRing_zpos=0.*cm, SkinClosingRing_zpos=0.*cm;
  ClosRgFoil_zpos=0.*cm;
  ClosRg_InRad=rInner[numZPlanes-1];
  ClosRg_zpos=InCondHalfLength+ClosRgDepth/2.;

  solidClosingRing=new G4Tubs("CLR1",ClosRg_InRad,OutCondRad+OutCondDepth,ClosRgDepth/2.,0.*deg,360.*deg);
  logicClosingRing=new G4LogicalVolume(solidClosingRing,HornMaterial,"CLR1");
  if(place){
    physiClosingRing=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,ClosRg_zpos+Horn_zpos),logicClosingRing,"CLR1",logicWorld,false,0);
    G4cout << physiClosingRing->GetName() <<" created."<<G4endl;
  }

}
//----------------------------------------------
void SBDetectorConstruction::DoInnerConductor(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductor "<< G4endl;  

  np=3;
  numZPlanes=5+np;

  for(int i=0;i<8;i++){
    zPlane[i]=0;
    rInner[i]=0;
    rOuter[i]=0;
  }

  zPlane[0]=-InCondHalfLength;
  zPlane[1]=InCondLength[0]-InCondHalfLength;
  zPlane[2]=InCondLength[0]+InCondLength[1]-InCondHalfLength;
  zPlane[3]=InCondLength[0]+InCondLength[1]+InCondLength[2]-InCondHalfLength;

  rInner[0]=InCondRad[0];
  rInner[1]=InCondRad[0];
  rInner[2]=InCondRad[0];
  rInner[3]=InCondRad[0];

  rOuter[0]=InCondRad[0]+InCondDepth[0];
  rOuter[1]=InCondRad[0]+InCondDepth[0];
  rOuter[2]=InCondRad[0]+InCondDepth[1];
  rOuter[3]=InCondRad[0]+InCondDepth[1];

  theta=acos(1.-InCondLength[4]/InCondCurvRad)/np;

  for(int i=0;i<np+1;i++){
    zPlane[4+i]=InCondHalfLength-InCondCurvRad*(1-cos((np-i)*theta));
    rInner[4+i]=InCondRad[1]+InCondCurvRad*(sin(np*theta)-sin((np-i)*theta));
    rOuter[4+i]=rInner[4+i]+InCondDepth[2];
    G4cout<<4+i<<" position z= "<<zPlane[4+i]<<" rI= "<<rInner[4+i]<<" rO = "<<rOuter[4+i]<<G4endl;
  }

  if(MINIBOONEHORN==false){
    G4cout << "ciccio -1 "<<MINIBOONEHORN<<G4endl;
    solidInnerConductor=new G4Polycone("INCO",0.*deg,360.*deg,numZPlanes,zPlane,rInner,rOuter);
  }

}

//----------------------------------------------
void SBDetectorConstruction::DoReflector(bool place)
{
  G4cout << "SBDetectorConstruction::DoReflector "<< G4endl;  
  for(int i=0;i<3;i++){
    zPl_RE[i]=0;
    rI_RE[i]=0;
    rO_RE[i]=0;
  }

  zPl_RE[0]=-0.5*ReflInLength;
  zPl_RE[1]=-0.5*ReflInLength+OutCondLength;
  zPl_RE[2]=0.5*ReflInLength;

  rI_RE[0]=OutCondRad+OutCondDepth;
  rI_RE[1]=OutCondRad+OutCondDepth;
  rI_RE[2]=ReflInForwardRad;

  rO_RE[0]=OutCondRad+OutCondDepth+ReflThickness;
  rO_RE[1]=OutCondRad+OutCondDepth+ReflThickness;
  rO_RE[2]=ReflInForwardRad+ReflThickness;

  //G4double Refl_zpos=OutCond_zpos-.5*OutCondLength+.5*ReflInLength;
  Refl_zpos=OutCond_zpos-.5*OutCondLength+.5*ReflInLength + MB2_DZ;// added manual shift

  solidRefl = new G4Polycone("REFI",0*deg,360*deg,3,zPl_RE,rI_RE,rO_RE);
  logicRefl=new G4LogicalVolume(solidRefl,ReflectorMaterial,"REFI");
  if(place){
    physiRefl=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Refl_zpos+Horn_zpos),logicRefl,"REFI",logicWorld,false,0);
    G4cout << physiRefl->GetName() <<" created."<<G4endl;
  }
}

//----------------------------------------------
void SBDetectorConstruction::DoTunnel(bool place)
{
  G4cout << "SBDetectorConstruction::DoTunnel "<< G4endl;  
  Tunnel_zpos=HalfMotherVoluLength;
  solidTunnel=new G4Tubs("TUNL",0.*cm,TunnelRadius,0.5*TunnelLength,0.*deg,360.*deg);
  logicTunnel=new G4LogicalVolume(solidTunnel,TunnelMaterial,"TUNL");
  if(place){
    physiTunnel=new G4PVPlacement(0,G4ThreeVector(0,0,Tunnel_zpos),logicTunnel,"TUNL",logicWorld,false,0);
    G4cout << physiTunnel->GetName() <<" created."<<G4endl;
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoTunnelPS(bool place)
{
  G4cout << "SBDetectorConstruction::DoTunnelPS "<<G4endl;
  //double Tunnel_zpos1=-5*m;
  //double Tunnel_zpos2=17.5*m;

  //double Tunnel_zpos1=-477.5*m;
  //double Tunnel_zpos2=-455.0*m;
  double Tunnel_zpos1=-477.5*m+3.15*m;//R.Santacesaria
  double Tunnel_zpos2=-455.0*m+3.15*m+14.6*m;//R.Santacesaria attention they must overlap a bit ! 
  //solidTunnelPS1=new G4Box("TUNLPS1",1.75*m,1.4*m,12.5*m);
  solidTunnelPS1=new G4Box("TUNLPS1",1.75*m,1.4*m,15.65*m);//R.Santacesaria
  logicTunnelPS1=new G4LogicalVolume(solidTunnelPS1,TunnelMaterial,"TUNLPS1");

  //solidTunnelPS2=new G4Box("TUNLPS2",2.5*m,1.4*m,10.*m);
  solidTunnelPS2=new G4Box("TUNLPS2",1.75*m,1.75*m,21.5*m);//R.Santacesaria
  logicTunnelPS2=new G4LogicalVolume(solidTunnelPS2,TunnelMaterial,"TUNLPS2");

  if(place){    
    physiTunnelPS1=new G4PVPlacement(0,G4ThreeVector(0,0,Tunnel_zpos1),logicTunnelPS1,"TUNLPS1",logicWorld,false,0);
    G4cout << physiTunnelPS1->GetName() <<" created."<<G4endl;
    physiTunnelPS2=new G4PVPlacement(0,G4ThreeVector(0,0,Tunnel_zpos2),logicTunnelPS2,"TUNLPS2",logicWorld,false,0);
    G4cout << physiTunnelPS2->GetName() <<" created."<<G4endl;
  }

}
//----------------------------------------------
void SBDetectorConstruction::DoReflTopCylinder(bool place)
{
  G4cout << "SBDetectorConstruction::DoReflTopCylinder "<< G4endl;  
  solidTopCylinder=new G4Tubs("REFO",ReflOutRad,ReflOutRad+ReflThickness,ReflInLength/2.,0.*deg,360.*deg);
  logicTopCylinder=new G4LogicalVolume(solidTopCylinder,ReflectorMaterial,"REFO");
  if(place){
    physiTopCylinder=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Refl_zpos+Horn_zpos),logicTopCylinder,"REFO",logicWorld,false,0);
    G4cout << physiTopCylinder->GetName() <<" created."<<G4endl;
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoTarget(bool place)
{
  G4cout << "SBDetectorConstruction::DoTarget "<< G4endl;  
  TargetZetaPos= TargetThickness/2.-InCondHalfLength-EndFlangeTotalDepth + Horn_zpos;
  if(MINIBOONEHORN)TargetZetaPos= TargetThickness/2.-0.5*MB_totL + Horn_zpos + InCond_zpos;
  if(MINOSHORN)TargetZetaPos=0.;// rule via control card

  Target_DZ=GetTarget_DZ();
  TargetZetaPos += Target_DZ;//manual shift

  G4cout << "Target manually shifted by " << Target_DZ/cm << " cm via mac card."<<G4endl; 
  G4cout << "Target starts in absolute frame at z = " << (TargetZetaPos-0.5*TargetThickness)/cm << " cm"<<G4endl; 
  G4cout << "Target starts (wrt inner conductor start) at z = " << (TargetZetaPos-0.5*TargetThickness-InCond_zpos-Horn_zpos)/cm << " cm"<<G4endl; 

  if(TargetThickness>0.){
    if(!MINOSHORN){
      solidTarget=new G4Tubs("TARG",0.*cm,TargetDiameter/2.,TargetThickness/2.,0.*deg,360.*deg);
      //logicTarget=new G4LogicalVolume(solidTarget,TargetMaterial,TargetMaterial->GetName());
      logicTarget=new G4LogicalVolume(solidTarget,TargetMaterial,"TARG");
      
      if(place){
	physiTarget[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,TargetZetaPos),logicTarget,"TARG",logicWorld,false,0);
	G4cout << physiTarget[0]->GetName() <<"[0] created."<<G4endl;
	if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
	  physiTarget[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,TargetZetaPos),logicTarget,"TARG",logicWorld,false,0);
	  G4cout << physiTarget[1]->GetName() <<"[1] created."<<G4endl;
	  physiTarget[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,TargetZetaPos),logicTarget,"TARG",logicWorld,false,0);
	  G4cout << physiTarget[2]->GetName() <<"[2] created."<<G4endl;
	  physiTarget[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,TargetZetaPos),logicTarget,"TARG",logicWorld,false,0);
	  G4cout << physiTarget[3]->GetName() <<"[3] created."<<G4endl;
	}else{
	  G4cout<< "SBDetectorConstruction::DoTarget WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
	}
      }//place
      
    } else if(MINOSHORN){
      solidTarget=new G4Tubs("TARG",0.*cm,TargetDiameter/2.,TargetThickness/2.,0.*deg,360.*deg);
      logicTarget=new G4LogicalVolume(solidTarget,TargetMaterial,"TARG");
      
      G4double z = TargetZetaPos;
      G4double gap = 9*cm;
      
      G4PVPlacement *physiTargetMS0=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      /*
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS1=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS2=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS3=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS4=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS5=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS6=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      z+=(TargetThickness+gap);
      G4PVPlacement *physiTargetMS7=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicTarget,"TARG",logicWorld,false,0);
      */
    }//MINOS HORN
    
    
  }// thickness>0


}


//----------------------------------------------
void SBDetectorConstruction::DoReflForwRing(bool place)
{
  G4cout << "SBDetectorConstruction::DoReflForwRing "<< G4endl;  
  ReflClRing_zpos=Refl_zpos+.5*ReflInLength+.5*ReflThickness;
  solidReflForwardRing=new G4Tubs("REFR",ReflInForwardRad,ReflOutRad+ReflThickness,ReflThickness/2.,0.*deg,360.*deg);
  logicReflForwardRing=new G4LogicalVolume(solidReflForwardRing,ReflectorMaterial,"REFR");
  if(place){
    physiReflForwardRing=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,ReflClRing_zpos+Horn_zpos),logicReflForwardRing,"REFR",logicWorld,false,0);
    G4cout << physiReflForwardRing->GetName() <<" created."<<G4endl;
  }

}

//----------------------------------------------
void SBDetectorConstruction::DoInnerConductor_MB1(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductor_MB1 "<< G4endl;  
  G4double MB1_zpos =InCond_zpos+Horn_zpos;
  rInnerMB_skin[NMB]=0;
  rOuterMB_skin[NMB]=0;
  zPlaneMB_skin[NMB]=0;

  for(int i=0;i<NMB;i++){
  //  rInnerMB_skin[i]=rInnerMB[i]-2.4;
    zPlaneMB_skin[i]=zPlaneMB[i];
  }

  /***************************************/
  double d=MB1_SkinThickness;
  zInnerPlaneMB_skin[0]=zPlaneMB[0]-d;//only valid if first angle is 90deg ...
  rInnerMB_skin[0]=rInnerMB[0];
  for (int i=0;i<NMB-1;i++){
    thetaI[i]=0.;
    if(zPlaneMB[i+1]!=zPlaneMB[i])thetaI[i]=TMath::ATan((rInnerMB[i+1]-rInnerMB[i])/(zPlaneMB[i+1]-zPlaneMB[i]));
  }
  thetaI[NMB-1]=TMath::ATan((rOuterMB[NMB-2]-rInnerMB[NMB-1])/(zPlaneMB[NMB-2]-zPlaneMB[NMB-1]))+TMath::Pi();
  //
  for (int i=0;i<NMB-1;i++){
    zInnerPlaneMB_skin[i+1]=zPlaneMB[i+1]+(d/TMath::Cos(0.5*(thetaI[i+1]-thetaI[i])))*TMath::Sin(0.5*(thetaI[i+1]+thetaI[i]));
    rInnerMB_skin[i+1]=rInnerMB[i+1]-(d/TMath::Cos(0.5*(thetaI[i+1]-thetaI[i])))*TMath::Cos(0.5*(thetaI[i+1]+thetaI[i]));
  }
  zInnerPlaneMB_skin[NMB-1]=zPlaneMB[NMB-1]+(d/TMath::Cos(0.5*(thetaI[NMB-1]-thetaI[NMB-2])))*TMath::Sin(0.5*(thetaI[NMB-1]+thetaI[NMB-2]));
  rInnerMB_skin[NMB-1]=rInnerMB[NMB-1]-(d/TMath::Cos(0.5*(thetaI[NMB-1]-thetaI[NMB-2])))*TMath::Cos(0.5*(thetaI[NMB-1]+thetaI[NMB-2]));
  //for(int i=0;i<NMB;i++){G4cout <<"** NOskin "<<zPlaneMB[i]<<" "<<rInnerMB[i]<<G4endl;}
  //for(int i=0;i<NMB;i++){G4cout <<"** YESskin "<<zInnerPlaneMB_skin[i]<<" "<<rInnerMB_skin[i]<<G4endl;}
  /***************************************/
  // endcaps
  solidBECAP1=new G4Tubs("BECAP1",rInnerMB[0],rOuterMB[0],0.5*d,0.*deg,360.*deg);
  logicBECAP1=new G4LogicalVolume(solidBECAP1,HornMaterial,"BECAP1");
  //
  solidFECAP1=new G4Tubs("FECAP1",rInnerMB[NMB-1],rOuterMB[NMB-1],0.5*d,0.*deg,360.*deg);
  logicFECAP1=new G4LogicalVolume(solidFECAP1,HornMaterial,"FECAP1");
  //

  solidInnerConductor_MB1=new G4Polycone("ICMB1",0.*deg,360.*deg,MBzplanes,zInnerPlaneMB_skin,rInnerMB_skin,rInnerMB);
  logicInnerConductor_MB1=new G4LogicalVolume(solidInnerConductor_MB1,HornMaterial,"ICMB1");
  if(place){

    physiInnerConductor_MB1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,MB1_zpos),logicInnerConductor_MB1,"ICMB1",logicWorld,false,0);
    G4cout <<" z at which inner magnetic field starts "<< (MB1_zpos+zPlane[0])/cm<<" cm "<<G4endl;
    G4cout << physiInnerConductor_MB1[0]->GetName() <<"[0] created."<<G4endl;
    physiBECAP1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,MB1_zpos+zPlaneMB[0]-0.5*d),logicBECAP1,"BECAP1",logicWorld,false,0);
    physiFECAP1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,MB1_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP1,"FECAP1",logicWorld,false,0);

    G4cout << physiBECAP1[0]->GetName() <<"[0] created."<<G4endl;
    G4cout << physiFECAP1[0]->GetName() <<"[0] created."<<G4endl;

    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      physiInnerConductor_MB1[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,MB1_zpos),logicInnerConductor_MB1,"ICMB1",logicWorld,false,0);
      physiBECAP1[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,MB1_zpos+zPlaneMB[0]-0.5*d),logicBECAP1,"BECAP1",logicWorld,false,0);
      physiFECAP1[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,MB1_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP1,"FECAP1",logicWorld,false,0);

      G4cout << physiInnerConductor_MB1[1]->GetName() <<"[1] created."<<G4endl;
      G4cout << physiBECAP1[1]->GetName() <<"[1] created."<<G4endl;

      physiInnerConductor_MB1[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,MB1_zpos),logicInnerConductor_MB1,"ICMB1",logicWorld,false,0);
      physiBECAP1[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,MB1_zpos+zPlaneMB[0]-0.5*d),logicBECAP1,"BECAP1",logicWorld,false,0);
      physiFECAP1[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,MB1_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP1,"FECAP1",logicWorld,false,0);

      G4cout << physiInnerConductor_MB1[2]->GetName() <<"[2] created."<<G4endl;
      G4cout << physiBECAP1[2]->GetName() <<"[2] created."<<G4endl;

      physiInnerConductor_MB1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,MB1_zpos),logicInnerConductor_MB1,"ICMB1",logicWorld,false,0);
      physiBECAP1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,MB1_zpos+zPlaneMB[0]-0.5*d),logicBECAP1,"BECAP1",logicWorld,false,0);
      physiFECAP1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,MB1_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP1,"FECAP1",logicWorld,false,0);

      G4cout << physiInnerConductor_MB1[3]->GetName() <<"[3] created."<<G4endl;
      G4cout << physiBECAP1[3]->GetName() <<"[3] created."<<G4endl;

    }else{
      G4cout<< "SBDetectorConstruction::DoInnerConductor_MB1 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }
  G4cout<<"HORNSHAPE "<<MBzplanes<<G4endl;
  for(int i=0;i<MBzplanes;i++){
    G4cout<<"HORNSHAPE_IC1 "<<zPlaneMB[i]<<" "<<zInnerPlaneMB_skin[i]<<" "<<rInnerMB[i]<<" "<<rInnerMB_skin[i]<<G4endl;
  }

}
//----------------------------------------------
void SBDetectorConstruction::DoOuterConductor_MB1(bool place)
{
  G4cout << "SBDetectorConstruction::DoOuterConductor_MB1 "<< G4endl;  
  G4double MB1_zpos =InCond_zpos+Horn_zpos;
  for(int i=0;i<NMB;i++){
    //rOuterMB_skin[i]=rOuterMB[i]+2.4;
  }
  /***************************************/
  double d=MB1_SkinThickness;
  zOuterPlaneMB_skin[0]=zPlaneMB[0]-d;//only valid if first angle is 90deg ...
  rOuterMB_skin[0]=rOuterMB[0];
  for (int i=0;i<NMB-1;i++){
    thetaO[i]=0.;
    if(zPlaneMB[i+1]!=zPlaneMB[i])thetaO[i]=TMath::ATan((rOuterMB[i+1]-rOuterMB[i])/(zPlaneMB[i+1]-zPlaneMB[i]));
    //G4cout <<"ciccio " <<thetaO[i]<<G4endl;
  }
  thetaO[NMB-1]=TMath::ATan((rOuterMB[NMB-2]-rOuterMB[NMB-1])/(zPlaneMB[NMB-2]-zPlaneMB[NMB-1]))-TMath::Pi();
  //G4cout <<"ciccio " <<thetaO[NMB-1]<<G4endl;
  for (int i=0;i<NMB-1;i++){
    zOuterPlaneMB_skin[i+1]=zPlaneMB[i+1]-(d/TMath::Cos(0.5*(thetaO[i+1]-thetaO[i])))*TMath::Sin(0.5*(thetaO[i+1]+thetaO[i]));
    rOuterMB_skin[i+1]=rOuterMB[i+1]+(d/TMath::Cos(0.5*(thetaO[i+1]-thetaO[i])))*TMath::Cos(0.5*(thetaO[i+1]+thetaO[i]));
  }
  zOuterPlaneMB_skin[NMB-1]=zInnerPlaneMB_skin[NMB-1];
  rOuterMB_skin[NMB-1]=rOuterMB[NMB-1];
  //for (int i=0;i<NMB;i++){G4cout <<"** NOskin "<<zPlaneMB[i]<<" "<<rOuterMB[i]<<G4endl;}
  //for (int i=0;i<NMB;i++){G4cout <<"** YESskin "<<zOuterPlaneMB_skin[i]<<" "<<rOuterMB_skin[i]<<G4endl;}
  /***************************************/
  //
  solidOuterConductor_MB1=new G4Polycone("OCMB1",0.*deg,360.*deg,MBzplanes,zOuterPlaneMB_skin,rOuterMB,rOuterMB_skin);
  logicOuterConductor_MB1=new G4LogicalVolume(solidOuterConductor_MB1,HornMaterial,"OCMB1");
  if(place){
    physiOuterConductor_MB1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,MB1_zpos),logicOuterConductor_MB1,"OCMB1",logicWorld,false,0);
    G4cout << physiOuterConductor_MB1[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      physiOuterConductor_MB1[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,MB1_zpos),logicOuterConductor_MB1,"OCMB1",logicWorld,false,0);
      G4cout << physiOuterConductor_MB1[1]->GetName() <<"[1] created."<<G4endl;
      physiOuterConductor_MB1[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,MB1_zpos),logicOuterConductor_MB1,"OCMB1",logicWorld,false,0);
      G4cout << physiOuterConductor_MB1[2]->GetName() <<"[2] created."<<G4endl;
      physiOuterConductor_MB1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,MB1_zpos),logicOuterConductor_MB1,"OCMB1",logicWorld,false,0);
      G4cout << physiOuterConductor_MB1[3]->GetName() <<"[3] created."<<G4endl;
    }else{
      G4cout<< "SBDetectorConstruction::DoOuterConductor_MB1 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }
  for(int i=0;i<MBzplanes;i++){
    G4cout<<"HORNSHAPE_OC1 "<<zPlaneMB[i]<<" "<<zOuterPlaneMB_skin[i]<<" "<<rOuterMB[i]<<" "<<rOuterMB_skin[i]<<G4endl;
  }

}
//----------------------------------------------
void SBDetectorConstruction::DoInnerConductor_MB2(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductor_MB2 "<< G4endl;  
  for(int i=0;i<NMB;i++){
    rInnerMB_skin[i]=rInnerMB[i]-MB2_SkinThickness;
    //zPlaneMB_skin[i]=zPlaneMB[i];
  }

  /***************************************/
  double d=MB2_SkinThickness;
  zInnerPlaneMB_skin[0]=zPlaneMB[0]-d;//only valid if first angle is 90deg ...
  rInnerMB_skin[0]=rInnerMB[0]-MB2_SkinThickness;
  for (int i=0;i<NMB-1;i++){
    thetaI[i]=0.;
    if(zPlaneMB[i+1]!=zPlaneMB[i])thetaI[i]=TMath::ATan((rInnerMB[i+1]-rInnerMB[i])/(zPlaneMB[i+1]-zPlaneMB[i]));
  }
  thetaI[NMB-1]=TMath::ATan((rOuterMB[NMB-2]-rInnerMB[NMB-1])/(zPlaneMB[NMB-2]-zPlaneMB[NMB-1]))+TMath::Pi();
  //
  for (int i=0;i<NMB-1;i++){
    zInnerPlaneMB_skin[i+1]=zPlaneMB[i+1]+(d/TMath::Cos(0.5*(thetaI[i+1]-thetaI[i])))*TMath::Sin(0.5*(thetaI[i+1]+thetaI[i]));
    rInnerMB_skin[i+1]=rInnerMB[i+1]-(d/TMath::Cos(0.5*(thetaI[i+1]-thetaI[i])))*TMath::Cos(0.5*(thetaI[i+1]+thetaI[i]));
  }
  zInnerPlaneMB_skin[NMB-1]=zPlaneMB[NMB-1]+(d/TMath::Cos(0.5*(thetaI[NMB-1]-thetaI[NMB-2])))*TMath::Sin(0.5*(thetaI[NMB-1]+thetaI[NMB-2]));
  rInnerMB_skin[NMB-1]=rInnerMB[NMB-1]-(d/TMath::Cos(0.5*(thetaI[NMB-1]-thetaI[NMB-2])))*TMath::Cos(0.5*(thetaI[NMB-1]+thetaI[NMB-2]));
  //for(int i=0;i<NMB;i++){G4cout <<"** NOskin "<<zPlaneMB[i]<<" "<<rInnerMB[i]<<G4endl;}
  //for(int i=0;i<NMB;i++){G4cout <<"** YESskin "<<zInnerPlaneMB_skin[i]<<" "<<rInnerMB_skin[i]<<G4endl;}
  /***************************************/
  // endcaps
  solidBECAP2=new G4Tubs("BECAP2",rInnerMB[0],rOuterMB[0],0.5*d,0.*deg,360.*deg);
  logicBECAP2=new G4LogicalVolume(solidBECAP2,HornMaterial,"BECAP2");
  //
  solidFECAP2=new G4Tubs("FECAP2",rInnerMB[NMB-1],rOuterMB[NMB-1],0.5*d,0.*deg,360.*deg);
  logicFECAP2=new G4LogicalVolume(solidFECAP2,HornMaterial,"FECAP2");

  solidInnerConductor_MB2=new G4Polycone("ICMB2",0.*deg,360.*deg,MBzplanes,zInnerPlaneMB_skin,rInnerMB_skin,rInnerMB);    
  //solidInnerConductor_MB2=new G4Polycone("ICMB2",0.*deg,360.*deg,MBzplanes,zPlaneMB_skin,rInnerMB_skin,rInnerMB);    
  logicInnerConductor_MB2=new G4LogicalVolume(solidInnerConductor_MB2,ReflectorMaterial,"ICMB2");
  
  if(place){
    physiInnerConductor_MB2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos),logicInnerConductor_MB2,"ICMB2",logicWorld,false,0);
    G4cout << physiInnerConductor_MB2[0]->GetName() <<"[0] created."<<G4endl;
    physiBECAP2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos+zPlaneMB[0]-0.5*d),logicBECAP2,"BECAP2",logicWorld,false,0);
    physiFECAP2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP2,"FECAP2",logicWorld,false,0);
    G4cout << physiBECAP2[0]->GetName() <<"[0] created."<<G4endl;
    G4cout << physiFECAP2[0]->GetName() <<"[0] created."<<G4endl;

    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      physiInnerConductor_MB2[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos),logicInnerConductor_MB2,"ICMB2",logicWorld,false,0);
      G4cout << physiInnerConductor_MB2[1]->GetName() <<"[1] created."<<G4endl;

      physiBECAP2[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos+zPlaneMB[0]-0.5*d),logicBECAP2,"BECAP2",logicWorld,false,0);
      physiFECAP2[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP2,"FECAP2",logicWorld,false,0);
      G4cout << physiBECAP2[1]->GetName() <<"[1] created."<<G4endl;
      G4cout << physiFECAP2[1]->GetName() <<"[1] created."<<G4endl;

      physiInnerConductor_MB2[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos),logicInnerConductor_MB2,"ICMB2",logicWorld,false,0);
      G4cout << physiInnerConductor_MB2[2]->GetName() <<"[2] created."<<G4endl;

      physiBECAP2[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos+zPlaneMB[0]-0.5*d),logicBECAP2,"BECAP2",logicWorld,false,0);
      physiFECAP2[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP2,"FECAP2",logicWorld,false,0);
      G4cout << physiBECAP2[2]->GetName() <<"[1] created."<<G4endl;
      G4cout << physiFECAP2[2]->GetName() <<"[1] created."<<G4endl;

      physiInnerConductor_MB2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos),logicInnerConductor_MB2,"ICMB2",logicWorld,false,0);
      G4cout << physiInnerConductor_MB2[3]->GetName() <<"[3] created."<<G4endl;

      physiBECAP2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos+zPlaneMB[0]-0.5*d),logicBECAP2,"BECAP2",logicWorld,false,0);
      physiFECAP2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos+zPlaneMB[NMB-1]+0.5*d),logicFECAP2,"FECAP2",logicWorld,false,0);
      G4cout << physiBECAP2[3]->GetName() <<"[3] created."<<G4endl;
      G4cout << physiFECAP2[3]->GetName() <<"[3] created."<<G4endl;

    }else{
      G4cout<< "SBDetectorConstruction::DoInnerConductor_MB2 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoOuterConductor_MB2(bool place)
{
  G4cout << "SBDetectorConstruction::DoOuterConductor_MB2 "<< G4endl;  
  for(int i=0;i<NMB;i++){
    //rOuterMB_skin[i]=rOuterMB[i]+MB2_SkinThickness;
  }
  /***************************************/
  double d=MB2_SkinThickness;
  zOuterPlaneMB_skin[0]=zPlaneMB[0]-d;//only valid if first angle is 90deg ...
  rOuterMB_skin[0]=rOuterMB[0]+MB2_SkinThickness;
  for (int i=0;i<NMB-1;i++){
    thetaO[i]=0.;
    if(zPlaneMB[i+1]!=zPlaneMB[i])thetaO[i]=TMath::ATan((rOuterMB[i+1]-rOuterMB[i])/(zPlaneMB[i+1]-zPlaneMB[i]));
    //G4cout <<"ciccio " <<thetaO[i]<<G4endl;
  }
  thetaO[NMB-1]=TMath::ATan((rOuterMB[NMB-2]-rOuterMB[NMB-1])/(zPlaneMB[NMB-2]-zPlaneMB[NMB-1]))-TMath::Pi();
  //G4cout <<"ciccio " <<thetaO[NMB-1]<<G4endl;
  //
  for (int i=0;i<NMB-1;i++){
    zOuterPlaneMB_skin[i+1]=zPlaneMB[i+1]-(d/TMath::Cos(0.5*(thetaO[i+1]-thetaO[i])))*TMath::Sin(0.5*(thetaO[i+1]+thetaO[i]));
    rOuterMB_skin[i+1]=rOuterMB[i+1]+(d/TMath::Cos(0.5*(thetaO[i+1]-thetaO[i])))*TMath::Cos(0.5*(thetaO[i+1]+thetaO[i]));
  }
  zOuterPlaneMB_skin[NMB-1]=zInnerPlaneMB_skin[NMB-1];
  rOuterMB_skin[NMB-1]=rOuterMB[NMB-1];
  //for (int i=0;i<NMB;i++){G4cout <<"** NOskin "<<zPlaneMB[i]<<" "<<rOuterMB[i]<<G4endl;}
  //for (int i=0;i<NMB;i++){G4cout <<"** YESskin "<<zOuterPlaneMB_skin[i]<<" "<<rOuterMB_skin[i]<<G4endl;}
  /***************************************/


  //solidOuterConductor_MB2=new G4Polycone("OCMB2",0.*deg,360.*deg,MBzplanes,zPlaneMB_skin,rOuterMB,rOuterMB_skin);    
  solidOuterConductor_MB2=new G4Polycone("OCMB2",0.*deg,360.*deg,MBzplanes,zOuterPlaneMB_skin,rOuterMB,rOuterMB_skin);    
  logicOuterConductor_MB2=new G4LogicalVolume(solidOuterConductor_MB2,ReflectorMaterial,"OCMB2");
  
  if(place){
    physiOuterConductor_MB2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Horn_zpos),logicOuterConductor_MB2,"OCMB2",logicWorld,false,0);
    G4cout << physiOuterConductor_MB2[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      physiOuterConductor_MB2[1]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Horn_zpos),logicOuterConductor_MB2,"OCMB2",logicWorld,false,0);
      G4cout << physiOuterConductor_MB2[1]->GetName() <<"[1] created."<<G4endl;
      physiOuterConductor_MB2[2]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Horn_zpos),logicOuterConductor_MB2,"OCMB2",logicWorld,false,0);
      G4cout << physiOuterConductor_MB2[2]->GetName() <<"[2] created."<<G4endl;
      physiOuterConductor_MB2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Horn_zpos),logicOuterConductor_MB2,"OCMB2",logicWorld,false,0);
      G4cout << physiOuterConductor_MB2[3]->GetName() <<"[3] created."<<G4endl;
    }else{
      G4cout<< "SBDetectorConstruction::DoOuterConductor_MB2 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }
}

//-------------------------------------------------------
//-------------------------------------------------------
//-------------------------------------------------------
// MINOS like horns
//----------------------------------------------
void SBDetectorConstruction::ReadHornPar_MS(G4int HN, G4String fnam)
{
  G4cout <<"Reading horn "<< HN <<" parameters from file "<<fnam<<G4endl;
  ifstream fin(fnam);
  const G4int NMAX =100;
  G4int N=0,ind;
  G4double zstart = 0;

  G4double z=0,ri=0,ro=0;
  //SkinDepth_MS=10.;//mm
  SkinDepth_MS1=MB1_SkinThickness;
  SkinDepth_MS2=MB2_SkinThickness;

  G4cout << "SBDetectorConstruction: MINOS like horns skin thick 1 = "<< SkinDepth_MS1/mm << " (mm) skin thick 2 = " << SkinDepth_MS2/mm <<" (mm)"<< G4endl;

  fin>>N;
  fin>>zstart;
  G4cout << "z_start = "<< zstart <<" N. of points "<<N<<G4endl;
  G4cout<<"i   z   r_i   r_o"<<G4endl;
  for (G4int i=0;i<N;i++){
    fin>>ind>>z>>ri>>ro;
    G4cout<<i<<" "<< z << " " << ri << " " << ro <<G4endl;
    if(HN==1){
      NMS1=N;
      Z0_MS1=zstart;
      zMS1[i]=z;
      riMS1[i]=ri;
      roMS1[i]=ro;
      zMS1_skin[i]=z;
      riMS1_skin[i]=ri-SkinDepth_MS1;
      roMS1_skin[i]=ro+SkinDepth_MS1;
    }
    if(HN==2){
      NMS2=N;
      Z0_MS2=zstart;
      zMS2[i]=z;
      riMS2[i]=ri;
      roMS2[i]=ro;
      zMS2_skin[i]=z;
      riMS2_skin[i]=ri-SkinDepth_MS2;
      roMS2_skin[i]=ro+SkinDepth_MS2;
    }
  }

  if(HN==1){
    Horn1ZetaStart=zstart+zMS1[0];
    Horn1ZetaEnd=zstart+zMS1[N-1];
    //G4cout<<"z_MAX for horn "<<HN<< " = "<<(zstart+zMS1[N-1])/m<<" (m)."<<G4endl;
    //G4cout<<"Distance btw horn "<<HN<< " end and tunnel entrance "<<
    //  ((Tunnel_zpos-0.5*TunnelLength)-(zstart+zMS1[N-1]))/m<<" (m) "<<G4endl;
    //G4cout<<"Distance btw horn "<<HN<< " start and tunnel entrance "<<
    //  ((Tunnel_zpos-0.5*TunnelLength)-(zstart+zMS1[0]))/m<<" (m) "<<G4endl;
    //G4cout<<"Distance btw target start and horn "<<HN<< " entrance "<<
    //  (-(TargetZetaPos-0.5*TargetThickness)+(zstart+zMS1[0]))/m<<" (m) "<<G4endl;
  }
  if(HN==2){
    Horn2ZetaStart=zstart+zMS2[0];
    Horn2ZetaEnd=zstart+zMS2[N-1];
    //G4cout<<"z_MAX for horn "<<HN<< " = "<<(zstart+zMS2[N-1])/m<<" (m)."<<G4endl;
    //G4cout<<"Distance btw horn "<<HN<< " end and tunnel entrance "<<
    //  ((Tunnel_zpos-0.5*TunnelLength)-(zstart+zMS2[N-1]))/m<<" (m) "<<G4endl;
    //G4cout<<"Distance btw horn "<<HN<< " start and tunnel entrance "<<
    //  ((Tunnel_zpos-0.5*TunnelLength)-(zstart+zMS2[0]))/m<<" (m) "<<G4endl;
    //G4cout<<"Distance btw target start and horn "<<HN<< " entrance "<<
    //  (-(TargetZetaPos-0.5*TargetThickness)+(zstart+zMS2[0]))/m<<" (m) "<<G4endl;
  }


}
void SBDetectorConstruction::DoMagField_MS1(bool place)
{
  G4double stepMinimum1=1.0e-3*mm;
  SBField* myField1[4];
  G4Mag_UsualEqRhs* iEquation1[4];
  G4MagIntegratorStepper* iStepper1[4];
  G4ChordFinder* iChordFinder1[4];
  G4FieldManager* mfieldMgr1[4];

  for(int i=0;i<Horn_N;i++){
    myField1[i]=new SBField(current1,i,this);
    iEquation1[i]=new G4Mag_UsualEqRhs(myField1[i]);
    iStepper1[i]=new G4ClassicalRK4(iEquation1[i]);
    iChordFinder1[i]=new G4ChordFinder(myField1[i],stepMinimum1,iStepper1[i]);
    mfieldMgr1[i]=new G4FieldManager(myField1[i],iChordFinder1[i]);
  }

  solidBField_MS1=new G4Polycone("BFL_MS1",0.*deg,360.*deg,NMS1,zMS1,riMS1,roMS1);
  for(int i=0;i<Horn_N;i++){
    logicBField_MS1[i]=new G4LogicalVolume(solidBField_MS1,FillingMaterial,Form("BFL_MS1_%d",i),mfieldMgr1[i],0,0);
  }
  if(place){
    physiBField_MS1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS1),logicBField_MS1[0],"BFL_MS1_0",logicWorld,false,0);
    G4cout << physiBField_MS1[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      if(InjSchema==1){//B field in all horns simultaneously
	physiBField_MS1[1]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Z0_MS1),logicBField_MS1[1],"BFL_MS1_1",logicWorld,false,0);
	G4cout << physiBField_MS1[1]->GetName() <<"[1] created."<<G4endl;
	physiBField_MS1[2]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Z0_MS1),logicBField_MS1[2],"BFL_MS1_2",logicWorld,false,0);
	G4cout << physiBField_MS1[2]->GetName() <<"[2] created."<<G4endl;
	physiBField_MS1[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Z0_MS1),logicBField_MS1[3],"BFL_MS1_3",logicWorld,false,0);
	G4cout << physiBField_MS1[3]->GetName() <<"[3] created."<<G4endl;
      }
    }else{
      G4cout<< "SBDetectorConstruction::DoMagField_MS1 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }

}
//----------------------------------------------
void SBDetectorConstruction::DoMagField_MS2(bool place)
{
  G4double stepMinimum2=1.0e-3*mm;
  SBField* myField2[4];
  G4Mag_UsualEqRhs* iEquation2[4];
  G4MagIntegratorStepper* iStepper2[4];
  G4ChordFinder* iChordFinder2[4];
  G4FieldManager* mfieldMgr2[4];

  for(int i=0;i<Horn_N;i++){
    myField2[i]=new SBField(current2,i,this);
    iEquation2[i]=new G4Mag_UsualEqRhs(myField2[i]);
    iStepper2[i]=new G4ClassicalRK4(iEquation2[i]);
    iChordFinder2[i]=new G4ChordFinder(myField2[i],stepMinimum2,iStepper2[i]);
    mfieldMgr2[i]=new G4FieldManager(myField2[i],iChordFinder2[i]);
  }

  solidBField_MS2=new G4Polycone("BFL_MS2",0.*deg,360.*deg,NMS2,zMS2,riMS2,roMS2);
  for(int i=0;i<Horn_N;i++){
    logicBField_MS2[i]=new G4LogicalVolume(solidBField_MS2,FillingMaterial,Form("BFL_MS2_%d",i),mfieldMgr2[i],0,0);
  }
  if(place){
    physiBField_MS2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS2),logicBField_MS2[0],"BFL_MS2_0",logicWorld,false,0);
    G4cout << physiBField_MS2[0]->GetName() <<"[0] created."<<G4endl;
    if(sqrt(pow(Horn_xpos,2)+pow(Horn_ypos,2))>MinHornDisplacRad){
      if(InjSchema==1){//B field in all horns simultaneously
	physiBField_MS2[1]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,Horn_ypos,Z0_MS2),logicBField_MS2[1],"BFL_MS2_1",logicWorld,false,0);
	G4cout << physiBField_MS2[1]->GetName() <<"[1] created."<<G4endl;
	physiBField_MS2[2]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,-Horn_ypos,Z0_MS2),logicBField_MS2[2],"BFL_MS2_2",logicWorld,false,0);
	G4cout << physiBField_MS2[2]->GetName() <<"[2] created."<<G4endl;
	physiBField_MS2[3]=new G4PVPlacement(0,G4ThreeVector(-Horn_xpos,-Horn_ypos,Z0_MS2),logicBField_MS2[3],"BFL_MS2_3",logicWorld,false,0);
	G4cout << physiBField_MS2[3]->GetName() <<"[3] created."<<G4endl;
      }
    }else{
      G4cout<< "SBDetectorConstruction::DoMagField_MS2 WARNING: displacement of Horns is smaller than "<<MinHornDisplacRad/cm<<" cm. Physics wolume not placed !!!! " <<G4endl;
    }
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoInnerConductor_MS1(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductor_MS1 "<< G4endl;  
  solidInnerConductor_MS1=new G4Polycone("ICMS1",0.*deg,360.*deg,NMS1,zMS1_skin,riMS1_skin,riMS1);
  logicInnerConductor_MS1=new G4LogicalVolume(solidInnerConductor_MS1,HornMaterial,"ICMS1");
  if(place){
    physiInnerConductor_MS1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS1),logicInnerConductor_MS1,"ICMS1",logicWorld,false,0);
    G4cout << physiInnerConductor_MS1[0]->GetName() <<"[0] created."<<G4endl;
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoOuterConductor_MS1(bool place)
{
  G4cout << "SBDetectorConstruction::DoOuterConductor_MS1 "<< G4endl;  
  solidOuterConductor_MS1=new G4Polycone("OCMS1",0.*deg,360.*deg,NMS1,zMS1_skin,roMS1,roMS1_skin);
  logicOuterConductor_MS1=new G4LogicalVolume(solidOuterConductor_MS1,HornMaterial,"OCMS1");
  if(place){
    physiOuterConductor_MS1[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS1),logicOuterConductor_MS1,"OCMS1",logicWorld,false,0);
    G4cout << physiOuterConductor_MS1[0]->GetName() <<"[0] created."<<G4endl;
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoInnerConductor_MS2(bool place)
{
  G4cout << "SBDetectorConstruction::DoInnerConductor_MS2 "<< G4endl;  
  solidInnerConductor_MS2=new G4Polycone("ICMS2",0.*deg,360.*deg,NMS2,zMS2_skin,riMS2_skin,riMS2);
  logicInnerConductor_MS2=new G4LogicalVolume(solidInnerConductor_MS2,HornMaterial,"ICMS2");
  if(place){
    physiInnerConductor_MS2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS2),logicInnerConductor_MS2,"ICMS2",logicWorld,false,0);
    G4cout << physiInnerConductor_MS2[0]->GetName() <<"[0] created."<<G4endl;
  }
}
//----------------------------------------------
void SBDetectorConstruction::DoOuterConductor_MS2(bool place)
{
  G4cout << "SBDetectorConstruction::DoOuterConductor_MS2 "<< G4endl;  
  solidOuterConductor_MS2=new G4Polycone("OCMS2",0.*deg,360.*deg,NMS2,zMS2_skin,roMS2,roMS2_skin);
  logicOuterConductor_MS2=new G4LogicalVolume(solidOuterConductor_MS2,HornMaterial,"OCMS2");
  if(place){
    physiOuterConductor_MS2[0]=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,Z0_MS2),logicOuterConductor_MS2,"OCMS2",logicWorld,false,0);
    G4cout << physiOuterConductor_MS2[0]->GetName() <<"[0] created."<<G4endl;
  }
}

//-------------------------------------------------------
//-------------------------------------------------------
//-------------------------------------------------------


//------------------------------
void SBDetectorConstruction::DefineMaterials()
{
  G4cout << "SBDetectorConstruction::DefineMaterials "<< G4endl;  
  G4String symbol;
  G4double a, z, density;
  G4int ncomponents, natoms;
  G4double fractionmass;
  //
  G4Element* H  = new G4Element("Hydrogen",symbol="H" , z= 1., a= 1.01*g/mole);
  G4Element* C  = new G4Element("Carbon"  ,symbol="C" , z= 6., a= 12.01*g/mole);
  G4Element* N  = new G4Element("Nitrogen",symbol="N" , z= 7., a= 14.01*g/mole);
  G4Element* O  = new G4Element("Oxygen"  ,symbol="O" , z= 8., a= 16.00*g/mole);
  G4Element* Si = new G4Element("Silicon",symbol="Si" , z= 14., a= 28.09*g/mole);
  //< andrea
  G4Element* Al = new G4Element("Aluminum",symbol="Al" , z= 13., a= 26.9815386*g/mole);
  G4Element* Be=new G4Element("Berillium",symbol="Be" , z= 4., a= 9.0121823*g/mole);
  //andrea>
  G4Material* Aluminum=new G4Material("Aluminum",z=13.,a=26.98*g/mole,density=2.700*g/cm3);

  G4Material* myGraphite=new G4Material("myGraphite",z=6.,a=12.01*g/mole,density=1.85*g/cm3);
  G4Material* myTantalum=new G4Material("myTantalum",z=73.,a=180.94788*g/mole,density=16.69*g/cm3);
  G4Material* myBerillium=new G4Material("myBerillium",z=4.,a=9.0121823*g/mole,density=2.700*g/cm3);

  G4Material* BeO=new G4Material("BeO", density= 3.02*g/cm3, ncomponents=2);
  BeO->AddElement(Be,natoms=1);
  BeO->AddElement(O,natoms=1);
 
  G4Material* H2O=new G4Material("Water", density= 1.000*g/cm3, ncomponents=2);
  H2O->AddElement(H,natoms=2);
  H2O->AddElement(O,natoms=1);
  // overwrite computed meanExcitationEnergy with ICRU recommended value 
  H2O->GetIonisation()->SetMeanExcitationEnergy(75.0*eV);
  //
  G4Material* Sci=new G4Material("Scintillator", density= 1.032*g/cm3, ncomponents=2);
  Sci->AddElement(C,natoms=9);
  Sci->AddElement(H,natoms=10);
  //
  G4Material* Myl=new G4Material("Mylar", density= 1.397*g/cm3, ncomponents=3);
  Myl->AddElement(C,natoms=10);
  Myl->AddElement(H,natoms=8);
  Myl->AddElement(O,natoms=4);
  //
  G4Material* SiO2=new G4Material("quartz",density= 2.200*g/cm3, ncomponents=2);
  SiO2->AddElement(Si,natoms=1);
  SiO2->AddElement(O,natoms=2);
  //
  // define a material from elements.   case 2: mixture by fractional mass
  //
  G4Material* Air=new G4Material("Air",density= 1.290*mg/cm3,ncomponents=2);
  Air->AddElement(N,fractionmass=0.7);
  Air->AddElement(O,fractionmass=0.3);
  // <andrea
  G4Material* AlBeMet=new G4Material("AlBeMet",density=0.2071*g/cm3,ncomponents=2);
  AlBeMet->AddElement(Al,fractionmass=38.*perCent);//check!
  AlBeMet->AddElement(Be,fractionmass=62.*perCent);//check!
  //andrea>
  //
  G4Material* CO2=new G4Material("CarbonicGas", density= 1.842*mg/cm3, ncomponents=2,
				   kStateGas, 325.*kelvin, 50.*atmosphere);
  CO2->AddElement(C,natoms=1);
  CO2->AddElement(O,natoms=2);
 
  G4Material* steam=new G4Material("WaterSteam",density=0.3*mg/cm3,ncomponents=1,kStateGas,500.*kelvin,2.*atmosphere);
  steam->AddMaterial(H2O,fractionmass=1.);

  G4Material* Vacuum=new G4Material("Galactic", z=1., a=1.01*g/mole,density= universe_mean_density,kStateGas, 2.73*kelvin, 3.e-18*pascal);

  G4Material* beam=new G4Material("Beam", density= 1.e-5*g/cm3, ncomponents=1,kStateGas, STP_Temperature, 2.e-2*bar);

  beam->AddMaterial(Air, fractionmass=1.);
  // rho=p/287.5/293 @ 20 Celsius p=130 Pa
  G4Material* AirTunnel=new G4Material("AirTunnel",density= 0.00154*g/cm3,ncomponents=1,kStateGas,STP_Temperature,0.001238*atmosphere);//~130 Pa as CNGS and WANF
  AirTunnel->AddMaterial(Air,fractionmass=1.);

  //if(0)
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  //default materials of the World
  //defaultMaterial=Vacuum;
  defaultMaterial=Air;
  //--------------------- 
  HornMaterial=Aluminum;
  ReflectorMaterial=Aluminum;
  
  FillingMaterial=Air;// Helium try !!
  //FillingMaterial=Vacuum;// NB for DESIGN phase ONLY!!!!!

  TunnelMaterial=AirTunnel;//try Helium also  !!
  //TunnelMaterial=Vacuum;
  //TunnelMaterial=Air;
  //--------------------- 
}

//----------------------------------------------
void SBDetectorConstruction::DoAbsorber(bool place)
{
  G4cout << "SBDetectorConstruction::DoAbsorber "<< G4endl;  
  AbsorberZetaPos=0.;// rule via control card
  AbsorberZetaPos += Absorber_DZ;//manual shift

  G4cout << "Absorber starts in absolute frame at z = " << (AbsorberZetaPos-0.5*AbsorberThickness)/cm << " cm"<<G4endl; 
  
  if(AbsorberThickness>0.){
    solidAbsorber=new G4Tubs("TARG",0.*cm,AbsorberDiameter/2.,AbsorberThickness/2.,0.*deg,360.*deg);
    logicAbsorber=new G4LogicalVolume(solidAbsorber,AbsorberMaterial,"TARG");
    G4double z = AbsorberZetaPos;
    G4PVPlacement *physiAbsorber=new G4PVPlacement(0,G4ThreeVector(Horn_xpos,Horn_ypos,z),logicAbsorber,"ABSOR",logicWorld,false,0);
  }// thickness>0
}
//----------------------------------------------
void SBDetectorConstruction::DoNear(bool place){
  G4cout << "SBDetectorConstruction::DoNear "<< G4endl;  
  double zn=0;//(TargetZetaPos-0.5*TargetThickness)/m+127.*m;
  double beamYoffset=1.*m;
  zn=-498.55*m+127.*m+6.*m;
  cout<<"znear "<<zn/m<<" m "<<endl;
  solidNear=new G4Box("NEAR",2.5*m,2.5*m,6.*m);
  logicNear=new G4LogicalVolume(solidNear,TunnelMaterial,"NEAR");
  G4PVPlacement *physiNear=new G4PVPlacement(0,G4ThreeVector(0.,beamYoffset,zn),logicNear,"NEAR",logicWorld,false,0);
}
//----------------------------------------------
void SBDetectorConstruction::DoFar(bool place){
  G4cout << "SBDetectorConstruction::DoFar "<< G4endl;
  double zf=0;//(TargetZetaPos-0.5*TargetThickness)/m+885.*m;
  zf=-498.55*m+885.*m+12.*m;
  cout<<"zfar "<<zf/m<<" m "<<endl;
  solidFar=new G4Box("FAR",5.*m,2.5*m,12.*m);
  logicFar=new G4LogicalVolume(solidFar,TunnelMaterial,"FAR");
  G4PVPlacement *physiFar=new G4PVPlacement(0,G4ThreeVector(0.,0.,zf),logicFar,"FAR",logicWorld,false,0);//885+27.5 (tunnel end)
}
//----------------------------------------------
void SBDetectorConstruction::DoNearSM(bool place){
  G4cout << "SBDetectorConstruction::DoNearSM "<< G4endl;  
  double zn=0;
  double beamYoffset=1.*m;
  zn=-498.55*m+127.*m+14.*m;
  cout<<"znear SM "<<zn/m<<" m "<<endl;
  solidNearSM=new G4Box("NEARSM",4.*m,2.5*m,1.*m);
  logicNearSM=new G4LogicalVolume(solidNearSM,TunnelMaterial,"NEARSM");
  G4PVPlacement *physiNearSM=new G4PVPlacement(0,G4ThreeVector(0.,beamYoffset,zn),logicNearSM,"NEARSM",logicWorld,false,0);
}
//----------------------------------------------
void SBDetectorConstruction::DoFarSM(bool place){
  G4cout << "SBDetectorConstruction::DoFarSM "<< G4endl;
  double zf=0;
  zf=-498.55*m+885.*m+26.*m;
  cout<<"zfar SM "<<zf/m<<" m "<<endl;
  solidFarSM=new G4Box("FAR",8.*m,2.5*m,1.*m);
  logicFarSM=new G4LogicalVolume(solidFarSM,TunnelMaterial,"FARSM");
  G4PVPlacement *physiFarSM=new G4PVPlacement(0,G4ThreeVector(0.,0.,zf),logicFarSM,"FARSM",logicWorld,false,0);
}
//----------------------------------------------

