#include "SBDetectorMessenger.hh"

#include "SBDetectorConstruction.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"

SBDetectorMessenger::SBDetectorMessenger(
                                           SBDetectorConstruction* SBDet)
:SBDetector(SBDet)
{ 
  SBDir = new G4UIdirectory("/SB/");
  SBDir->SetGuidance("UI commands of this example");
  
  detDir = new G4UIdirectory("/SB/det/");
  detDir->SetGuidance("detector control");

  OffAxisCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setOA",this);
  OffAxisCmd->SetGuidance("Set Off Axis Angle (rad)");
  OffAxisCmd->SetParameterName("OffAxisAngle",false);
  OffAxisCmd->SetRange("OffAxisAngle>=0.");
  OffAxisCmd->SetUnitCategory("Angle");
  OffAxisCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  OffAxisPhiCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setOAphi",this);
  OffAxisPhiCmd->SetGuidance("Set Off Axis azimuthal angle (rad)");
  OffAxisPhiCmd->SetParameterName("OffAxisPhi",false);
  //OffAxisPhiCmd->SetRange("OffAxisPhi>=0.");
  OffAxisPhiCmd->SetUnitCategory("Angle");
  OffAxisPhiCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  TargMaterCmd = new G4UIcmdWithAString("/SB/det/setTargMat",this);
  TargMaterCmd->SetGuidance("Select Material of the Target.");
  TargMaterCmd->SetParameterName("choice",false);
  TargMaterCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  TargThickCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setTargThick",this);
  TargThickCmd->SetGuidance("Set Thickness of the Target");
  TargThickCmd->SetParameterName("Size",false);
  TargThickCmd->SetRange("Size>=0.");
  TargThickCmd->SetUnitCategory("Length");
  TargThickCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  DiameterCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setTargetDiameter",this);
  DiameterCmd->SetGuidance("Set tranverse size of the calorimeter");
  DiameterCmd->SetParameterName("Size",false);
  DiameterCmd->SetRange("Size>0.");
  DiameterCmd->SetUnitCategory("Length");    
  DiameterCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  AbsoMaterCmd = new G4UIcmdWithAString("/SB/det/setAbsoMat",this);
  AbsoMaterCmd->SetGuidance("Select Material of the absorber.");
  AbsoMaterCmd->SetParameterName("choice",false);
  AbsoMaterCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  AbsoThickCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setAbsoThick",this);
  AbsoThickCmd->SetGuidance("Set Thickness of the Absorber");
  AbsoThickCmd->SetParameterName("Size",false);
  AbsoThickCmd->SetRange("Size>=0.");
  AbsoThickCmd->SetUnitCategory("Length");
  AbsoThickCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  AbsoDiameterCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setAbsoDiam",this);
  AbsoDiameterCmd->SetGuidance("Set tranverse size of the absorber");
  AbsoDiameterCmd->SetParameterName("Size",false);
  AbsoDiameterCmd->SetRange("Size>0.");
  AbsoDiameterCmd->SetUnitCategory("Length");    
  AbsoDiameterCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  AbsoDZCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Abso_DZ",this);
  AbsoDZCmd->SetGuidance("Set Absorber parameter DZ");
  AbsoDZCmd->SetParameterName("Size",false);
  //AbsoDZCmd->SetRange("Size>=0.");
  AbsoDZCmd->SetUnitCategory("Length");
  AbsoDZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  TunLenCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setTunLen",this);
  TunLenCmd->SetGuidance("Set the length of the decay tunnel");
  TunLenCmd->SetParameterName("Size",false);
  TunLenCmd->SetRange("Size>=0.");
  TunLenCmd->SetUnitCategory("Length");
  TunLenCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  TunRadCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setTunRad",this);
  TunRadCmd->SetGuidance("Set the radius of the decay tunnel");
  TunRadCmd->SetParameterName("Size",false);
  TunRadCmd->SetRange("Size>0.");
  TunRadCmd->SetUnitCategory("Length");    
  TunRadCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  SkinThick1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/SkinThick1",this);
  SkinThick1Cmd->SetGuidance("Set the thickness of the skin for horn1");
  SkinThick1Cmd->SetParameterName("Size",false);
  SkinThick1Cmd->SetRange("Size>=0.");
  SkinThick1Cmd->SetUnitCategory("Length");
  SkinThick1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  SkinThick2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/SkinThick2",this);
  SkinThick2Cmd->SetGuidance("Set the thickness of the skin for horn2");
  SkinThick2Cmd->SetParameterName("Size",false);
  SkinThick2Cmd->SetRange("Size>=0.");
  SkinThick2Cmd->SetUnitCategory("Length");
  SkinThick2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  //
  // horn
  //
  HornDesCmd = new G4UIcmdWithAnInteger("/SB/det/HornDesign",this);
  HornDesCmd->SetGuidance("Set number of layers.");
  HornDesCmd->SetParameterName("HornDesign",false);
  //HornDesCmd->SetRange("");
  HornDesCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornInjCmd = new G4UIcmdWithAnInteger("/SB/det/Injection",this);
  HornInjCmd->SetGuidance("Switch proton injection schema (0=one by one, 1=all at once)");
  HornInjCmd->SetParameterName("Injection",false);
  //HornInjCmd->SetRange("");
  HornInjCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornSkinCmd = new G4UIcmdWithAnInteger("/SB/det/CondSkin",this);
  HornSkinCmd->SetGuidance("(0=only B field, 1=B field+conductor)");
  HornSkinCmd->SetParameterName("Skin",false);
  //HornSkinCmd->SetRange("");
  HornSkinCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornReflCmd = new G4UIcmdWithAnInteger("/SB/det/Reflector",this);
  HornReflCmd->SetGuidance("(0=no reflector, 1=reflector)");
  HornReflCmd->SetParameterName("Reflector",false);
  //HornReflCmd->SetRange("");
  HornReflCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornNCmd = new G4UIcmdWithAnInteger("/SB/det/Horn_N",this);
  HornNCmd->SetGuidance("Set number of horns.");
  HornNCmd->SetParameterName("Horn_N",false);
  //HornNCmd->SetRange("");
  HornNCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornL1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_L1",this);
  HornL1Cmd->SetGuidance("Set Horn parameter L1");
  HornL1Cmd->SetParameterName("Size",false);
  HornL1Cmd->SetRange("Size>=0.");
  HornL1Cmd->SetUnitCategory("Length");
  HornL1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornL2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_L2",this);
  HornL2Cmd->SetGuidance("Set Horn parameter L2");
  HornL2Cmd->SetParameterName("Size",false);
  HornL2Cmd->SetRange("Size>=0.");
  HornL2Cmd->SetUnitCategory("Length");
  HornL2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornL3Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_L3",this);
  HornL3Cmd->SetGuidance("Set Horn parameter L3");
  HornL3Cmd->SetParameterName("Size",false);
  HornL3Cmd->SetRange("Size>=0.");
  HornL3Cmd->SetUnitCategory("Length");
  HornL3Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornL4Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_L4",this);
  HornL4Cmd->SetGuidance("Set Horn parameter L4");
  HornL4Cmd->SetParameterName("Size",false);
  //HornL4Cmd->SetRange("Size>=0.");
  HornL4Cmd->SetUnitCategory("Length");
  HornL4Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornL5Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_L5",this);
  HornL5Cmd->SetGuidance("Set Horn parameter L5");
  HornL5Cmd->SetParameterName("Size",false);
  HornL5Cmd->SetRange("Size>=0.");
  HornL5Cmd->SetUnitCategory("Length");
  HornL5Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornrCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_r",this);
  HornrCmd->SetGuidance("Set Horn parameter r");
  HornrCmd->SetParameterName("Size",false);
  HornrCmd->SetRange("Size>=0.");
  HornrCmd->SetUnitCategory("Length");
  HornrCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornRCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_R",this);
  HornRCmd->SetGuidance("Set Horn parameter R");
  HornRCmd->SetParameterName("Size",false);
  HornRCmd->SetRange("Size>=0.");
  HornRCmd->SetUnitCategory("Length");
  HornRCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornR0Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_R0",this);
  HornR0Cmd->SetGuidance("Set Horn parameter R0");
  HornR0Cmd->SetParameterName("Size",false);
  HornR0Cmd->SetRange("Size>=0.");
  HornR0Cmd->SetUnitCategory("Length");
  HornR0Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornR1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_R1",this);
  HornR1Cmd->SetGuidance("Set Horn parameter R1");
  HornR1Cmd->SetParameterName("Size",false);
  HornR1Cmd->SetRange("Size>=0.");
  HornR1Cmd->SetUnitCategory("Length");
  HornR1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornR2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_R2",this);
  HornR2Cmd->SetGuidance("Set Horn parameter R2");
  HornR2Cmd->SetParameterName("Size",false);
  HornR2Cmd->SetRange("Size>=0.");
  HornR2Cmd->SetUnitCategory("Length");
  HornR2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornI1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_I1",this);
  HornI1Cmd->SetGuidance("Set Horn parameter I1");
  HornI1Cmd->SetParameterName("Current",false);
  //HornI1Cmd->SetRange("Current>=0.");
  //HornI1Cmd->SetUnitCategory("current");// Current no
  HornI1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornI2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_I2",this);
  HornI2Cmd->SetGuidance("Set Horn parameter I2");
  HornI2Cmd->SetParameterName("Current",false);
  //HornI2Cmd->SetRange("Current>=0.");
  //HornI2Cmd->SetUnitCategory("Current");
  HornI2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornXCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_X",this);
  HornXCmd->SetGuidance("Set Horn parameter X");
  HornXCmd->SetParameterName("Current",false);
  HornXCmd->SetUnitCategory("Length");
  HornXCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornYCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn_Y",this);
  HornYCmd->SetGuidance("Set Horn parameter Y");
  HornYCmd->SetParameterName("Current",false);
  HornYCmd->SetUnitCategory("Length");
  HornYCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2L1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_L1",this);
  Horn2L1Cmd->SetGuidance("Set Horn2 parameter L1");
  Horn2L1Cmd->SetParameterName("Size",false);
  Horn2L1Cmd->SetRange("Size>=0.");
  Horn2L1Cmd->SetUnitCategory("Length");
  Horn2L1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2L2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_L2",this);
  Horn2L2Cmd->SetGuidance("Set Horn2 parameter L2");
  Horn2L2Cmd->SetParameterName("Size",false);
  Horn2L2Cmd->SetRange("Size>=0.");
  Horn2L2Cmd->SetUnitCategory("Length");
  Horn2L2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2L3Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_L3",this);
  Horn2L3Cmd->SetGuidance("Set Horn2 parameter L3");
  Horn2L3Cmd->SetParameterName("Size",false);
  Horn2L3Cmd->SetRange("Size>=0.");
  Horn2L3Cmd->SetUnitCategory("Length");
  Horn2L3Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2L4Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_L4",this);
  Horn2L4Cmd->SetGuidance("Set Horn2 parameter L4");
  Horn2L4Cmd->SetParameterName("Size",false);
  //Horn2L4Cmd->SetRange("Size>=0.");
  Horn2L4Cmd->SetUnitCategory("Length");
  Horn2L4Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2L5Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_L5",this);
  Horn2L5Cmd->SetGuidance("Set Horn2 parameter L5");
  Horn2L5Cmd->SetParameterName("Size",false);
  Horn2L5Cmd->SetRange("Size>=0.");
  Horn2L5Cmd->SetUnitCategory("Length");
  Horn2L5Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2rCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_r",this);
  Horn2rCmd->SetGuidance("Set Horn2 parameter r");
  Horn2rCmd->SetParameterName("Size",false);
  Horn2rCmd->SetRange("Size>=0.");
  Horn2rCmd->SetUnitCategory("Length");
  Horn2rCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2RCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_R",this);
  Horn2RCmd->SetGuidance("Set Horn2 parameter R");
  Horn2RCmd->SetParameterName("Size",false);
  Horn2RCmd->SetRange("Size>=0.");
  Horn2RCmd->SetUnitCategory("Length");
  Horn2RCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2R0Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_R0",this);
  Horn2R0Cmd->SetGuidance("Set Horn2 parameter R0");
  Horn2R0Cmd->SetParameterName("Size",false);
  Horn2R0Cmd->SetRange("Size>=0.");
  Horn2R0Cmd->SetUnitCategory("Length");
  Horn2R0Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2R1Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_R1",this);
  Horn2R1Cmd->SetGuidance("Set Horn2 parameter R1");
  Horn2R1Cmd->SetParameterName("Size",false);
  Horn2R1Cmd->SetRange("Size>=0.");
  Horn2R1Cmd->SetUnitCategory("Length");
  Horn2R1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2R2Cmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_R2",this);
  Horn2R2Cmd->SetGuidance("Set Horn2 parameter R2");
  Horn2R2Cmd->SetParameterName("Size",false);
  Horn2R2Cmd->SetRange("Size>=0.");
  Horn2R2Cmd->SetUnitCategory("Length");
  Horn2R2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Horn2DZCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Horn2_DZ",this);
  Horn2DZCmd->SetGuidance("Set Horn2 parameter DZ");
  Horn2DZCmd->SetParameterName("Size",false);
  //Horn2DZCmd->SetRange("Size>=0.");
  Horn2DZCmd->SetUnitCategory("Length");
  Horn2DZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  TargetDZCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/Target_DZ",this);
  TargetDZCmd->SetGuidance("Set Target parameter DZ");
  TargetDZCmd->SetParameterName("Size",false);
  //TargetDZCmd->SetRange("Size>=0.");
  TargetDZCmd->SetUnitCategory("Length");
  TargetDZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornFile1Cmd = new G4UIcmdWithAString("/SB/det/SetFileHorn1",this);
  HornFile1Cmd->SetGuidance("Select file with horn shape.");
  HornFile1Cmd->SetParameterName("horn file",false);
  HornFile1Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  HornFile2Cmd = new G4UIcmdWithAString("/SB/det/SetFileHorn2",this);
  HornFile2Cmd->SetGuidance("Select file with horn shape.");
  HornFile2Cmd->SetParameterName("horn file",false);
  HornFile2Cmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  UpdateCmd = new G4UIcmdWithoutParameter("/SB/det/update",this);
  UpdateCmd->SetGuidance("Update calorimeter geometry.");
  UpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  UpdateCmd->SetGuidance("if you changed geometrical value(s).");
  UpdateCmd->AvailableForStates(G4State_Idle);
      
  //MagFieldCmd = new G4UIcmdWithADoubleAndUnit("/SB/det/setField",this);  
  //MagFieldCmd->SetGuidance("Define magnetic field.");
  //MagFieldCmd->SetGuidance("Magnetic field will be in Z direction.");
  //MagFieldCmd->SetParameterName("Bz",false);
  //MagFieldCmd->SetUnitCategory("Magnetic flux density");
  //MagFieldCmd->AvailableForStates(G4State_PreInit,G4State_Idle);  
}


SBDetectorMessenger::~SBDetectorMessenger()
{

  delete OffAxisCmd;
  delete OffAxisPhiCmd;

  delete TargMaterCmd;
  delete TargThickCmd;
  delete DiameterCmd;   
  delete UpdateCmd;

  delete TunLenCmd;
  delete TunRadCmd;

  delete SkinThick1Cmd;
  delete SkinThick2Cmd;

  delete HornDesCmd;
  delete HornInjCmd;
  delete HornSkinCmd;
  delete HornReflCmd;
  delete HornNCmd;

  delete HornL1Cmd;
  delete HornL2Cmd;
  delete HornL3Cmd;
  delete HornL4Cmd;
  delete HornL5Cmd;
  delete HornrCmd;
  delete HornRCmd;
  delete HornR0Cmd;
  delete HornR1Cmd;
  delete HornR2Cmd;

  delete HornI1Cmd;
  delete HornI2Cmd;

  delete HornXCmd;
  delete HornYCmd;

  delete Horn2L1Cmd;
  delete Horn2L2Cmd;
  delete Horn2L3Cmd;
  delete Horn2L4Cmd;
  delete Horn2L5Cmd;
  delete Horn2rCmd;
  delete Horn2RCmd;
  delete Horn2R0Cmd;
  delete Horn2R1Cmd;
  delete Horn2R2Cmd;

  delete HornFile1Cmd;
  delete HornFile2Cmd;

  delete Horn2DZCmd;
  delete TargetDZCmd;
  delete AbsoDZCmd;

  delete detDir;
  delete SBDir;  
}


void SBDetectorMessenger::SetNewValue(G4UIcommand* command,G4String newValue)
{ 

  if( command == OffAxisPhiCmd )
   { SBDetector->SetOffAxisPhi(OffAxisPhiCmd->GetNewDoubleValue(newValue));}

  if( command == OffAxisCmd )
   { SBDetector->SetOffAxisAngle(OffAxisCmd->GetNewDoubleValue(newValue));}

  if( command == TargMaterCmd )
   { SBDetector->SetTargetMaterial(newValue);}
   
  if( command == TargThickCmd )
   { SBDetector->SetTargetThickness(TargThickCmd->GetNewDoubleValue(newValue));}
   
  if( command == DiameterCmd )
   { SBDetector->SetTargetDiameter(DiameterCmd->GetNewDoubleValue(newValue));}


  //

  if( command == AbsoMaterCmd )
   { SBDetector->SetAbsorberMaterial(newValue);}
   
  if( command == AbsoThickCmd )
   { SBDetector->SetAbsorberThickness(TargThickCmd->GetNewDoubleValue(newValue));}
   
  if( command == AbsoDiameterCmd )
   { SBDetector->SetAbsorberDiameter(DiameterCmd->GetNewDoubleValue(newValue));}

  //
  
  if( command == TunLenCmd )
   { SBDetector->SetTunnelLength(TunLenCmd->GetNewDoubleValue(newValue));}
   
  if( command == TunRadCmd )
   { SBDetector->SetTunnelRadius(TunRadCmd->GetNewDoubleValue(newValue));}

  if( command == SkinThick1Cmd )
   { SBDetector->SetMB1_SkinThickness(SkinThick1Cmd->GetNewDoubleValue(newValue));}

  if( command == SkinThick2Cmd )
   { SBDetector->SetMB2_SkinThickness(SkinThick2Cmd->GetNewDoubleValue(newValue));}

  if( command == HornDesCmd )
   { SBDetector->SetHornDesign(HornDesCmd->GetNewIntValue(newValue));}

  if( command == HornSkinCmd )
   { SBDetector->SetSkin(HornSkinCmd->GetNewIntValue(newValue));}

  if( command == HornReflCmd )
   { SBDetector->SetRefl(HornReflCmd->GetNewIntValue(newValue));}

  if( command == HornInjCmd )
   { SBDetector->SetInj(HornInjCmd->GetNewIntValue(newValue));}

  if( command == HornNCmd )
   { SBDetector->SetHornNumber(HornNCmd->GetNewIntValue(newValue));}
  
  if(command==HornL1Cmd){SBDetector->SetHorn_L1(HornL1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornL2Cmd){SBDetector->SetHorn_L2(HornL1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornL3Cmd){SBDetector->SetHorn_L3(HornL1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornL4Cmd){SBDetector->SetHorn_L4(HornL1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornL5Cmd){SBDetector->SetHorn_L5(HornL1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornrCmd){SBDetector->SetHorn_r(HornrCmd->GetNewDoubleValue(newValue));}
  if(command==HornRCmd){SBDetector->SetHorn_R(HornRCmd->GetNewDoubleValue(newValue));}
  if(command==HornR0Cmd){SBDetector->SetHorn_R0(HornR0Cmd->GetNewDoubleValue(newValue));}
  if(command==HornR1Cmd){SBDetector->SetHorn_R1(HornR1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornR2Cmd){SBDetector->SetHorn_R2(HornR2Cmd->GetNewDoubleValue(newValue));}

  if(command==HornI1Cmd){SBDetector->SetHorn_I1(HornI1Cmd->GetNewDoubleValue(newValue));}
  if(command==HornI2Cmd){SBDetector->SetHorn_I2(HornI2Cmd->GetNewDoubleValue(newValue));}

  if(command==HornXCmd){SBDetector->SetHornX(HornXCmd->GetNewDoubleValue(newValue));}
  if(command==HornYCmd){SBDetector->SetHornY(HornYCmd->GetNewDoubleValue(newValue));}

  if(command==Horn2L1Cmd){SBDetector->SetHorn2_L1(Horn2L1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2L2Cmd){SBDetector->SetHorn2_L2(Horn2L1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2L3Cmd){SBDetector->SetHorn2_L3(Horn2L1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2L4Cmd){SBDetector->SetHorn2_L4(Horn2L1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2L5Cmd){SBDetector->SetHorn2_L5(Horn2L1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2rCmd){SBDetector->SetHorn2_r(Horn2rCmd->GetNewDoubleValue(newValue));}
  if(command==Horn2RCmd){SBDetector->SetHorn2_R(Horn2RCmd->GetNewDoubleValue(newValue));}
  if(command==Horn2R0Cmd){SBDetector->SetHorn2_R0(Horn2R0Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2R1Cmd){SBDetector->SetHorn2_R1(Horn2R1Cmd->GetNewDoubleValue(newValue));}
  if(command==Horn2R2Cmd){SBDetector->SetHorn2_R2(Horn2R2Cmd->GetNewDoubleValue(newValue));}

  if(command==Horn2DZCmd){SBDetector->SetHorn2_DZ(Horn2DZCmd->GetNewDoubleValue(newValue));}

  if(command==TargetDZCmd){SBDetector->SetTarget_DZ(TargetDZCmd->GetNewDoubleValue(newValue));}

  if(command==AbsoDZCmd){SBDetector->SetAbsorber_DZ(AbsoDZCmd->GetNewDoubleValue(newValue));}

  if(command==HornFile1Cmd){SBDetector->SetHornFile1(newValue);}
  if(command==HornFile2Cmd){SBDetector->SetHornFile2(newValue);}

  if( command == UpdateCmd )
   { SBDetector->UpdateGeometry(); }

  //if( command == MagFieldCmd )
  //{ SBDetector->SetMagField(MagFieldCmd->GetNewDoubleValue(newValue));}
}
